from __future__ import annotations

from typing import Any, Optional

from .utils import *

class RNNCell:
    def call(self, inputs: Any, states: Any, **kwargs: Any) -> None: ...
    def __call__(self, inputs: Any, states: Any, **kwargs: Any): ...
    shape: Any = ...
    def get_initial_states(
        self,
        batch_ref: Any,
        shape: Optional[Any] = ...,
        dtype: str = ...,
        init_value: int = ...,
        batch_dim_idx: int = ...,
    ): ...
    @property
    def state_shape(self) -> None: ...
    @property
    def state_dtype(self) -> None: ...

class GRUCell(RNNCell):
    hidden_size: Any = ...
    gru_unit: Any = ...
    def __init__(
        self,
        hidden_size: Any,
        param_attr: Optional[Any] = ...,
        bias_attr: Optional[Any] = ...,
        gate_activation: Optional[Any] = ...,
        activation: Optional[Any] = ...,
        dtype: str = ...,
        name: str = ...,
    ) -> None: ...
    def call(self, inputs: Any, states: Any): ...
    @property
    def state_shape(self): ...

class LSTMCell(RNNCell):
    hidden_size: Any = ...
    lstm_unit: Any = ...
    def __init__(
        self,
        hidden_size: Any,
        param_attr: Optional[Any] = ...,
        bias_attr: Optional[Any] = ...,
        gate_activation: Optional[Any] = ...,
        activation: Optional[Any] = ...,
        forget_bias: float = ...,
        dtype: str = ...,
        name: str = ...,
    ) -> None: ...
    def call(self, inputs: Any, states: Any): ...
    @property
    def state_shape(self): ...

def rnn(
    cell: Any,
    inputs: Any,
    initial_states: Optional[Any] = ...,
    sequence_length: Optional[Any] = ...,
    time_major: bool = ...,
    is_reverse: bool = ...,
    **kwargs: Any,
): ...

class ArrayWrapper:
    array: Any = ...
    def __init__(self, x: Any) -> None: ...
    def append(self, x: Any): ...
    def __getitem__(self, item: Any): ...

def birnn(
    cell_fw: Any,
    cell_bw: Any,
    inputs: Any,
    initial_states: Optional[Any] = ...,
    sequence_length: Optional[Any] = ...,
    time_major: bool = ...,
    **kwargs: Any,
): ...

class Decoder:
    def initialize(self, inits: Any) -> None: ...
    def step(self, time: Any, inputs: Any, states: Any, **kwargs: Any) -> None: ...
    def finalize(self, outputs: Any, final_states: Any, sequence_lengths: Any) -> None: ...
    @property
    def tracks_own_finished(self): ...

class BeamSearchDecoder(Decoder):
    cell: Any = ...
    embedding_fn: Any = ...
    output_fn: Any = ...
    start_token: Any = ...
    end_token: Any = ...
    beam_size: Any = ...
    def __init__(
        self,
        cell: Any,
        start_token: Any,
        end_token: Any,
        beam_size: Any,
        embedding_fn: Optional[Any] = ...,
        output_fn: Optional[Any] = ...,
    ) -> None: ...
    @staticmethod
    def tile_beam_merge_with_batch(x: Any, beam_size: Any): ...

    class OutputWrapper: ...
    class StateWrapper: ...
    kinf: float = ...
    batch_size: Any = ...
    start_token_tensor: Any = ...
    end_token_tensor: Any = ...
    def initialize(self, initial_cell_states: Any): ...
    def step(self, time: Any, inputs: Any, states: Any, **kwargs: Any): ...
    def finalize(self, outputs: Any, final_states: Any, sequence_lengths: Any): ...
    @property
    def tracks_own_finished(self): ...

def dynamic_decode(
    decoder: Any,
    inits: Optional[Any] = ...,
    max_step_num: Optional[Any] = ...,
    output_time_major: bool = ...,
    impute_finished: bool = ...,
    is_test: bool = ...,
    return_length: bool = ...,
    **kwargs: Any,
): ...

class DecodeHelper:
    def initialize(self) -> None: ...
    def sample(self, time: Any, outputs: Any, states: Any) -> None: ...
    def next_inputs(self, time: Any, outputs: Any, states: Any, sample_ids: Any) -> None: ...

class TrainingHelper(DecodeHelper):
    inputs: Any = ...
    sequence_length: Any = ...
    time_major: Any = ...
    inputs_: Any = ...
    def __init__(self, inputs: Any, sequence_length: Any, time_major: bool = ...): ...
    def initialize(self): ...
    def sample(self, time: Any, outputs: Any, states: Any): ...
    def next_inputs(self, time: Any, outputs: Any, states: Any, sample_ids: Any): ...

class GreedyEmbeddingHelper(DecodeHelper):
    embedding_fn: Any = ...
    start_tokens: Any = ...
    end_token: Any = ...
    def __init__(self, embedding_fn: Any, start_tokens: Any, end_token: Any) -> None: ...
    def initialize(self): ...
    def sample(self, time: Any, outputs: Any, states: Any): ...
    def next_inputs(self, time: Any, outputs: Any, states: Any, sample_ids: Any): ...

class SampleEmbeddingHelper(GreedyEmbeddingHelper):
    softmax_temperature: Any = ...
    seed: Any = ...
    def __init__(
        self,
        embedding_fn: Any,
        start_tokens: Any,
        end_token: Any,
        softmax_temperature: Optional[Any] = ...,
        seed: Optional[Any] = ...,
    ) -> None: ...
    def sample(self, time: Any, outputs: Any, states: Any): ...

class BasicDecoder(Decoder):
    cell: Any = ...
    helper: Any = ...
    output_fn: Any = ...
    def __init__(self, cell: Any, helper: Any, output_fn: Optional[Any] = ...) -> None: ...
    def initialize(self, initial_cell_states: Any): ...

    class OutputWrapper: ...

    def step(self, time: Any, inputs: Any, states: Any, **kwargs: Any): ...

def dynamic_lstm(
    input: Any,
    size: Any,
    h_0: Optional[Any] = ...,
    c_0: Optional[Any] = ...,
    param_attr: Optional[Any] = ...,
    bias_attr: Optional[Any] = ...,
    use_peepholes: bool = ...,
    is_reverse: bool = ...,
    gate_activation: str = ...,
    cell_activation: str = ...,
    candidate_activation: str = ...,
    dtype: str = ...,
    name: Optional[Any] = ...,
): ...
def lstm(
    input: Any,
    init_h: Any,
    init_c: Any,
    max_len: Any,
    hidden_size: Any,
    num_layers: Any,
    dropout_prob: float = ...,
    is_bidirec: bool = ...,
    is_test: bool = ...,
    name: Optional[Any] = ...,
    default_initializer: Optional[Any] = ...,
    seed: int = ...,
): ...
def dynamic_lstmp(
    input: Any,
    size: Any,
    proj_size: Any,
    param_attr: Optional[Any] = ...,
    bias_attr: Optional[Any] = ...,
    use_peepholes: bool = ...,
    is_reverse: bool = ...,
    gate_activation: str = ...,
    cell_activation: str = ...,
    candidate_activation: str = ...,
    proj_activation: str = ...,
    dtype: str = ...,
    name: Optional[Any] = ...,
    h_0: Optional[Any] = ...,
    c_0: Optional[Any] = ...,
    cell_clip: Optional[Any] = ...,
    proj_clip: Optional[Any] = ...,
): ...
def dynamic_gru(
    input: Any,
    size: Any,
    param_attr: Optional[Any] = ...,
    bias_attr: Optional[Any] = ...,
    is_reverse: bool = ...,
    gate_activation: str = ...,
    candidate_activation: str = ...,
    h_0: Optional[Any] = ...,
    origin_mode: bool = ...,
): ...
def gru_unit(
    input: Any,
    hidden: Any,
    size: Any,
    param_attr: Optional[Any] = ...,
    bias_attr: Optional[Any] = ...,
    activation: str = ...,
    gate_activation: str = ...,
    origin_mode: bool = ...,
): ...
def beam_search(
    pre_ids: Any,
    pre_scores: Any,
    ids: Any,
    scores: Any,
    beam_size: Any,
    end_id: Any,
    level: int = ...,
    is_accumulated: bool = ...,
    name: Optional[Any] = ...,
    return_parent_idx: bool = ...,
): ...
def beam_search_decode(ids: Any, scores: Any, beam_size: Any, end_id: Any, name: Optional[Any] = ...): ...
def lstm_unit(
    x_t: Any,
    hidden_t_prev: Any,
    cell_t_prev: Any,
    forget_bias: float = ...,
    param_attr: Optional[Any] = ...,
    bias_attr: Optional[Any] = ...,
    name: Optional[Any] = ...,
): ...



from __future__ import annotations

from typing import Any

from .. import core as core
from ..framework import Variable as Variable
from ..framework import static_only as static_only
from ..framework import unique_name as unique_name
from .control_flow import array_length as array_length
from .control_flow import array_write as array_write
from .layer_function_generator import OpProtoHolder as OpProtoHolder

compare_ops: Any
EXPRESSION_MAP: Any

def monkey_patch_variable(): ...



from __future__ import annotations

from typing import Any

def create_graph(optimizer_list: Any): ...
def topo_sort(edge: Any, indegree: Any): ...
def floyd(edge: Any): ...
def maximum_path_len_algo(optimizer_list: Any): ...

class StrategyCompilerBase:
    def __init__(self) -> None: ...

class StrategyCompiler(StrategyCompilerBase):
    def __init__(self) -> None: ...
    def generate_optimizer(
        self,
        loss: Any,
        role_maker: Any,
        optimizer: Any,
        user_defined_strategy: Any,
        meta_optimizer_list: Any,
        graph_optimizer_list: Any,
    ): ...



from __future__ import annotations

from typing import Any, Optional

from .. import Layer as Layer

class BCEWithLogitsLoss(Layer):
    weight: Any = ...
    reduction: Any = ...
    pos_weight: Any = ...
    name: Any = ...
    def __init__(
        self,
        weight: Optional[Any] = ...,
        reduction: str = ...,
        pos_weight: Optional[Any] = ...,
        name: Optional[Any] = ...,
    ) -> None: ...
    def forward(self, logit: Any, label: Any): ...

class CrossEntropyLoss(Layer):
    weight: Any = ...
    reduction: Any = ...
    ignore_index: Any = ...
    soft_label: Any = ...
    axis: Any = ...
    use_softmax: Any = ...
    name: Any = ...
    def __init__(
        self,
        weight: Optional[Any] = ...,
        ignore_index: int = ...,
        reduction: str = ...,
        soft_label: bool = ...,
        axis: int = ...,
        use_softmax: bool = ...,
        name: Optional[Any] = ...,
    ) -> None: ...
    def forward(self, input: Any, label: Any): ...

class HSigmoidLoss(Layer):
    weight: Any = ...
    bias: Any = ...
    def __init__(
        self,
        feature_size: Any,
        num_classes: Any,
        weight_attr: Optional[Any] = ...,
        bias_attr: Optional[Any] = ...,
        is_custom: bool = ...,
        is_sparse: bool = ...,
        name: Optional[Any] = ...,
    ) -> None: ...
    def forward(self, input: Any, label: Any, path_table: Optional[Any] = ..., path_code: Optional[Any] = ...): ...

class MSELoss(Layer):
    reduction: Any = ...
    def __init__(self, reduction: str = ...) -> None: ...
    def forward(self, input: Any, label: Any): ...

class L1Loss(Layer):
    reduction: Any = ...
    name: Any = ...
    def __init__(self, reduction: str = ..., name: Optional[Any] = ...) -> None: ...
    def forward(self, input: Any, label: Any): ...

class BCELoss(Layer):
    weight: Any = ...
    reduction: Any = ...
    name: Any = ...
    def __init__(self, weight: Optional[Any] = ..., reduction: str = ..., name: Optional[Any] = ...) -> None: ...
    def forward(self, input: Any, label: Any): ...

class NLLLoss(Layer):
    def __init__(
        self, weight: Optional[Any] = ..., ignore_index: int = ..., reduction: str = ..., name: Optional[Any] = ...
    ) -> None: ...
    def forward(self, input: Any, label: Any): ...

class KLDivLoss(Layer):
    reduction: Any = ...
    def __init__(self, reduction: str = ...) -> None: ...
    def forward(self, input: Any, label: Any): ...

class MarginRankingLoss(Layer):
    margin: Any = ...
    reduction: Any = ...
    name: Any = ...
    def __init__(self, margin: float = ..., reduction: str = ..., name: Optional[Any] = ...) -> None: ...
    def forward(self, input: Any, other: Any, label: Any): ...

class CTCLoss(Layer):
    blank: Any = ...
    reduction: Any = ...
    def __init__(self, blank: int = ..., reduction: str = ...) -> None: ...
    def forward(
        self, log_probs: Any, labels: Any, input_lengths: Any, label_lengths: Any, norm_by_times: bool = ...
    ): ...

class SmoothL1Loss(Layer):
    reduction: Any = ...
    delta: Any = ...
    name: Any = ...
    def __init__(self, reduction: str = ..., delta: float = ..., name: Optional[Any] = ...) -> None: ...
    def forward(self, input: Any, label: Any): ...

class HingeEmbeddingLoss(Layer):
    margin: Any = ...
    reduction: Any = ...
    name: Any = ...
    def __init__(self, margin: float = ..., reduction: str = ..., name: Optional[Any] = ...) -> None: ...
    def forward(self, input: Any, label: Any): ...



from __future__ import annotations

from collections import OrderedDict
from typing import Any

class LRUSharedCache(OrderedDict):
    limit: int = ...
    def __init__(self) -> None: ...
    def get(self, key: Any): ...
    def __setitem__(self, key: Any, value: Any) -> None: ...

shared_cache: Any

def cuda_from_cache(key: Any): ...
def rebuild_tensor(cls, lodtensor: Any, metadata: Any): ...
def reduce_tensor(tensor: Any): ...
def rebuild_lodtensor_filename(cls, ipc_name: Any, size: Any, type_idx: Any, dims: Any, lod: Any): ...
def rebuild_cuda_tensor(
    cls, handle: Any, offset_bytes: Any, size: Any, type_idx: Any, dims: Any, lod: Any, device_idx: Any
): ...
def rebuild_lodtensor_empty(cls): ...
def reduce_lodtensor(lodtensor: Any): ...
def init_reductions() -> None: ...



from __future__ import annotations

from typing import Any, Callable, Optional

from .._typing import NPUPlace, _DTypeNumpy, dtype
from . import core

def in_dygraph_mode(): ...
def ipu_shard_guard(index: Optional[Any] = ..., stage: Optional[Any] = ...) -> None: ...
def require_version(min_version: Any, max_version: Optional[Any] = ...): ...
def is_compiled_with_xpu(): ...
def is_compiled_with_npu(): ...
def is_compiled_with_cinn(): ...
def is_compiled_with_cuda(): ...
def is_compiled_with_rocm(): ...
def cuda_places(device_ids: Optional[Any] = ...): ...
def xpu_places(device_ids: Optional[Any] = ...): ...
def npu_places(device_ids: Optional[list[int]] = ...) -> list[NPUPlace]: ...
def cpu_places(device_count: Optional[Any] = ...): ...
def cuda_pinned_places(device_count: Optional[Any] = ...): ...
def mlu_places(device_ids: Optional[Any] = ...): ...
def dygraph_not_support(func: Callable[..., Any]) -> Callable[..., Any]: ...
def dygraph_only(func: Callable[..., Any]) -> Callable[..., Any]: ...
def static_only(func: Callable[..., Any]) -> Callable[..., Any]: ...
def fake_interface_only(func: Callable[..., Any]) -> Callable[..., Any]: ...
def convert_np_dtype_to_dtype_(np_dtype: _DTypeNumpy) -> dtype: ...
def disable_signal_handler() -> None: ...
def monkey_patch_math_varbase() -> None: ...

class NameScope:
    def __init__(self, name: str = ..., parent: Optional[Any] = ...) -> None: ...
    def child(self, prefix: Any): ...
    def parent(self): ...
    def name(self): ...

def name_scope(prefix: Optional[Any] = ...) -> None: ...

class VariableMetaClass(type):
    @classmethod
    def __instancecheck__(cls, instance: Any): ...

class ParameterMetaClass(VariableMetaClass):
    @classmethod
    def __instancecheck__(cls, instance: Any): ...

class Variable(metaclass=VariableMetaClass):
    block: Any = ...
    belong_to_optimizer: Any = ...
    error_clip: Any = ...
    desc: Any = ...
    op: Any = ...
    is_data: Any = ...
    def __init__(
        self,
        block: Any,
        type: Any = ...,
        name: Optional[Any] = ...,
        shape: Optional[Any] = ...,
        dtype: Optional[Any] = ...,
        lod_level: Optional[Any] = ...,
        capacity: Optional[Any] = ...,
        persistable: Optional[Any] = ...,
        error_clip: Optional[Any] = ...,
        stop_gradient: bool = ...,
        is_data: bool = ...,
        need_check_feed: bool = ...,
        belong_to_optimizer: bool = ...,
        **kwargs: Any,
    ) -> None: ...
    def detach(self): ...
    def numpy(self) -> None: ...
    def backward(self, retain_graph: bool = ...) -> None: ...
    def gradient(self) -> None: ...
    def clear_gradient(self) -> None: ...
    def register_hook(self, hook: Any) -> None: ...
    def to_string(self, throw_on_error: Any, with_details: bool = ...): ...
    def element_size(self): ...
    @property
    def stop_gradient(self): ...
    @stop_gradient.setter
    def stop_gradient(self, s: Any) -> None: ...
    @property
    def persistable(self): ...
    @persistable.setter
    def persistable(self, p: Any) -> None: ...
    @property
    def is_parameter(self): ...
    @is_parameter.setter
    def is_parameter(self, p: Any) -> None: ...
    @property
    def name(self): ...
    @property
    def grad_name(self): ...
    @name.setter
    def name(self, new_name: Any) -> None: ...
    @property
    def shape(self): ...
    @property
    def dtype(self): ...
    @property
    def lod_level(self): ...
    @property
    def type(self): ...
    @property
    def T(self): ...
    def clone(self): ...
    def __getitem__(self, item: Any): ...
    def __setitem__(self, item: Any, value: Any): ...
    def get_value(self, scope: Optional[Any] = ...): ...
    def set_value(self, value: Any, scope: Optional[Any] = ...) -> None: ...
    def size(self): ...
    @property
    def attr_names(self): ...
    @property
    def process_mesh(self): ...
    @property
    def shard_mask(self): ...
    @property
    def offload_device(self): ...

class OpProtoHolder:
    @classmethod
    def instance(cls): ...
    op_proto_map: Any = ...
    def __init__(self) -> None: ...
    def get_op_proto(self, type: Any): ...
    def update_op_proto(self): ...
    @staticmethod
    def generated_op_attr_names(): ...

class Operator:
    OP_WITHOUT_KERNEL_SET: Any = ...
    attrs: Any = ...
    block: Any = ...
    desc: Any = ...
    def __init__(
        self,
        block: Any,
        desc: Any,
        type: Optional[Any] = ...,
        inputs: Optional[Any] = ...,
        outputs: Optional[Any] = ...,
        attrs: Optional[Any] = ...,
    ): ...
    def to_string(self, throw_on_error: Any): ...
    @property
    def type(self): ...
    def input(self, name: Any): ...
    @property
    def input_names(self): ...
    @property
    def input_arg_names(self): ...
    @property
    def output_arg_names(self): ...
    def output(self, name: Any): ...
    @property
    def output_names(self): ...
    @property
    def idx(self): ...
    def has_attr(self, name: Any): ...
    def attr_type(self, name: Any): ...
    @property
    def attr_names(self): ...
    def attr(self, name: Any): ...
    def all_attrs(self): ...
    @property
    def process_mesh(self): ...
    def dims_mapping(self, name: Any): ...
    @property
    def pipeline_stage(self): ...

class Block:
    desc: Any = ...
    vars: Any = ...
    ops: Any = ...
    program: Any = ...
    removed_vars: Any = ...
    def __init__(self, program: Any, idx: Any) -> None: ...
    def to_string(self, throw_on_error: Any, with_details: bool = ...): ...
    @property
    def parent_idx(self): ...
    @property
    def forward_block_idx(self): ...
    @property
    def backward_block_idx(self): ...
    @property
    def idx(self): ...
    def var(self, name: Any): ...
    def all_parameters(self): ...
    def iter_parameters(self): ...
    def create_var(self, *args: Any, **kwargs: Any): ...
    def has_var(self, name: Any): ...
    def create_parameter(self, *args: Any, **kwargs: Any): ...
    def append_op(self, *args: Any, **kwargs: Any): ...

class IrNode:
    node: Any = ...
    def __init__(self, node: Any) -> None: ...
    def name(self): ...
    def node_type(self): ...
    def var(self): ...
    def op(self): ...
    def id(self): ...
    def is_op(self): ...
    def is_var(self): ...
    def is_ctrl_var(self): ...
    def clear_inputs(self) -> None: ...
    def remove_input_by_id(self, node_id: Any) -> None: ...
    def remove_input(self, node: Any) -> None: ...
    def append_input(self, node: Any) -> None: ...
    def clear_outputs(self) -> None: ...
    def remove_output_by_id(self, node_id: Any) -> None: ...
    def remove_output(self, node: Any) -> None: ...
    def append_output(self, node: Any) -> None: ...
    @property
    def inputs(self): ...
    @property
    def outputs(self): ...

class IrVarNode(IrNode):
    node: Any = ...
    def __init__(self, node: Any) -> None: ...
    def set_shape(self, shape: Any) -> None: ...
    def persistable(self): ...
    def type(self): ...
    def dtype(self): ...
    def shape(self): ...
    @property
    def inputs(self): ...
    @property
    def outputs(self): ...

class IrOpNode(IrNode):
    node: Any = ...
    def __init__(self, node: Any) -> None: ...
    def rename_input(self, old_input_name: Any, new_input_name: Any) -> None: ...
    def rename_output(self, old_output_name: Any, new_output_name: Any) -> None: ...
    def input(self, name: Any): ...
    def output(self, name: Any): ...
    def set_type(self, new_type: Any): ...
    def set_attr(self, name: Any, val: Any) -> None: ...
    def input_arg_names(self): ...
    def output_arg_names(self): ...
    @property
    def inputs(self): ...
    @property
    def outputs(self): ...

class IrGraph:
    graph: Any = ...
    def __init__(self, graph: Any, for_test: bool = ...) -> None: ...
    def clone(self): ...
    def is_test(self): ...
    def all_nodes(self): ...
    def all_var_nodes(self): ...
    def all_persistable_nodes(self): ...
    def all_op_nodes(self): ...
    def all_sub_graphs(self, for_test: bool = ...): ...
    def get_sub_graph(self, i: Any, for_test: bool = ...): ...
    def create_persistable_node(self, name: Any, var_type: Any, shape: Any, var_dtype: Any): ...
    def create_var_node(self, name: Any, var_type: Any, shape: Any, var_dtype: Any): ...
    def create_control_dep_var(self): ...
    def create_var_node_from_desc(self, var_desc: Any): ...
    def create_op_node(self, op_type: Any, attrs: Any, inputs: Any, outputs: Any): ...
    def create_op_node_from_desc(self, op_desc: Any): ...
    def update_input_link(self, old_input_node: Any, new_input_node: Any, op_node: Any) -> None: ...
    def update_output_link(self, old_output_node: Any, new_output_node: Any, op_node: Any) -> None: ...
    def link_to(self, node_in: Any, node_out: Any) -> None: ...
    def safe_remove_nodes(self, remove_nodes: Any) -> None: ...
    def resolve_hazard(self) -> None: ...
    def has_circle(self): ...
    def graph_num(self): ...
    def topology_sort(self): ...
    def build_adjacency_list(self): ...
    def draw(
        self, save_path: Any, name: Any, marked_nodes: Optional[Any] = ..., remove_ctr_var: bool = ...
    ) -> None: ...
    def to_program(self): ...

class Program:
    desc: Any = ...
    blocks: Any = ...
    current_block_idx: int = ...
    def __init__(self) -> None: ...
    def global_seed(self, seed: int = ...) -> None: ...
    def to_string(self, throw_on_error: Any, with_details: bool = ...): ...
    def clone(self, for_test: bool = ...): ...
    @staticmethod
    def parse_from_string(binary_str: Any): ...
    @property
    def random_seed(self): ...
    @property
    def num_blocks(self): ...
    @random_seed.setter
    def random_seed(self, seed: Any) -> None: ...
    def global_block(self): ...
    def block(self, index: Any): ...
    def current_block(self): ...
    def list_vars(self) -> None: ...
    def all_parameters(self): ...
    def state_dict(self, mode: str = ..., scope: Optional[Any] = ...): ...
    def set_state_dict(self, state_dict: Any, scope: Optional[Any] = ...) -> None: ...

class Parameter(Variable, metaclass=ParameterMetaClass):
    trainable: Any = ...
    optimize_attr: Any = ...
    regularizer: Any = ...
    do_model_average: Any = ...
    need_clip: Any = ...
    is_distributed: bool = ...
    is_parameter: bool = ...
    def __init__(self, block: Any, shape: Any, dtype: Any, type: Any = ..., **kwargs: Any) -> None: ...
    def to_string(self, throw_on_error: Any, with_details: bool = ...): ...

class ParamBase(core.VarBase):
    stop_gradient: Any = ...
    optimize_attr: Any = ...
    regularizer: Any = ...
    do_model_average: Any = ...
    need_clip: Any = ...
    is_distributed: Any = ...
    def __init__(self, shape: Any, dtype: Any, **kwargs: Any) -> None: ...
    @property
    def trainable(self): ...
    @trainable.setter
    def trainable(self, trainable: Any) -> None: ...
    def __deepcopy__(self, memo: Any): ...

class EagerParamBase(_core_eager_eagertensor):
    stop_gradient: Any = ...
    optimize_attr: Any = ...
    regularizer: Any = ...
    do_model_average: Any = ...
    need_clip: Any = ...
    is_distributed: Any = ...
    def __init__(self, shape: Any, dtype: Any, **kwargs: Any) -> None: ...
    @property
    def trainable(self): ...
    @trainable.setter
    def trainable(self, trainable: Any) -> None: ...
    def __deepcopy__(self, memo: Any): ...

def default_startup_program(): ...
def default_main_program(): ...
def program_guard(main_program: Any, startup_program: Optional[Any] = ...) -> None: ...
def device_guard(device: Optional[Any] = ...) -> None: ...
def set_flags(flags: Any) -> None: ...
def get_flags(flags: Any): ...

# Names in __all__ with no definition:
#   _non_static_mode



from __future__ import annotations

from typing import Any, Optional

from paddle.io import Dataset

def has_valid_extension(filename: Any, extensions: Any): ...
def make_dataset(dir: Any, class_to_idx: Any, extensions: Any, is_valid_file: Optional[Any] = ...): ...

class DatasetFolder(Dataset):
    root: Any = ...
    transform: Any = ...
    loader: Any = ...
    extensions: Any = ...
    classes: Any = ...
    class_to_idx: Any = ...
    samples: Any = ...
    targets: Any = ...
    dtype: Any = ...
    def __init__(
        self,
        root: Any,
        loader: Optional[Any] = ...,
        extensions: Optional[Any] = ...,
        transform: Optional[Any] = ...,
        is_valid_file: Optional[Any] = ...,
    ) -> None: ...
    def __getitem__(self, index: Any): ...
    def __len__(self): ...

IMG_EXTENSIONS: Any

def pil_loader(path: Any): ...
def cv2_loader(path: Any): ...
def default_loader(path: Any): ...

class ImageFolder(Dataset):
    root: Any = ...
    loader: Any = ...
    extensions: Any = ...
    samples: Any = ...
    transform: Any = ...
    def __init__(
        self,
        root: Any,
        loader: Optional[Any] = ...,
        extensions: Optional[Any] = ...,
        transform: Optional[Any] = ...,
        is_valid_file: Optional[Any] = ...,
    ): ...
    def __getitem__(self, index: Any): ...
    def __len__(self): ...



from __future__ import annotations

from . import bf16 as bf16
from . import decorator as decorator
from . import fp16_lists as fp16_lists
from . import fp16_utils as fp16_utils
from .decorator import *
from .fp16_lists import *
from .fp16_utils import *



from __future__ import annotations

from typing import Any, Optional

from paddle.distribution import distribution
from paddle.fluid import core as core
from paddle.fluid.data_feeder import check_dtype as check_dtype
from paddle.fluid.data_feeder import (
    check_variable_and_dtype as check_variable_and_dtype,
)
from paddle.fluid.framework import in_dygraph_mode as in_dygraph_mode
from paddle.fluid.layers import control_flow as control_flow
from paddle.fluid.layers import elementwise_mul as elementwise_mul

class Normal(distribution.Distribution):
    batch_size_unknown: bool = ...
    all_arg_is_float: bool = ...
    name: Any = ...
    dtype: str = ...
    loc: Any = ...
    scale: Any = ...
    def __init__(self, loc: Any, scale: Any, name: Optional[Any] = ...) -> None: ...
    def sample(self, shape: Any, seed: int = ...): ...
    def entropy(self): ...
    def log_prob(self, value: Any): ...
    def probs(self, value: Any): ...
    def kl_divergence(self, other: Any): ...



from __future__ import annotations

from typing import Any, Optional

def fft(x: Any, n: Optional[Any] = ..., axis: int = ..., norm: str = ..., name: Optional[Any] = ...): ...
def ifft(x: Any, n: Optional[Any] = ..., axis: int = ..., norm: str = ..., name: Optional[Any] = ...): ...
def rfft(x: Any, n: Optional[Any] = ..., axis: int = ..., norm: str = ..., name: Optional[Any] = ...): ...
def irfft(x: Any, n: Optional[Any] = ..., axis: int = ..., norm: str = ..., name: Optional[Any] = ...): ...
def hfft(x: Any, n: Optional[Any] = ..., axis: int = ..., norm: str = ..., name: Optional[Any] = ...): ...
def ihfft(x: Any, n: Optional[Any] = ..., axis: int = ..., norm: str = ..., name: Optional[Any] = ...): ...
def fftn(x: Any, s: Optional[Any] = ..., axes: Optional[Any] = ..., norm: str = ..., name: Optional[Any] = ...): ...
def ifftn(x: Any, s: Optional[Any] = ..., axes: Optional[Any] = ..., norm: str = ..., name: Optional[Any] = ...): ...
def rfftn(x: Any, s: Optional[Any] = ..., axes: Optional[Any] = ..., norm: str = ..., name: Optional[Any] = ...): ...
def irfftn(x: Any, s: Optional[Any] = ..., axes: Optional[Any] = ..., norm: str = ..., name: Optional[Any] = ...): ...
def hfftn(x: Any, s: Optional[Any] = ..., axes: Optional[Any] = ..., norm: str = ..., name: Optional[Any] = ...): ...
def ihfftn(x: Any, s: Optional[Any] = ..., axes: Optional[Any] = ..., norm: str = ..., name: Optional[Any] = ...): ...
def fft2(x: Any, s: Optional[Any] = ..., axes: Any = ..., norm: str = ..., name: Optional[Any] = ...): ...
def ifft2(x: Any, s: Optional[Any] = ..., axes: Any = ..., norm: str = ..., name: Optional[Any] = ...): ...
def rfft2(x: Any, s: Optional[Any] = ..., axes: Any = ..., norm: str = ..., name: Optional[Any] = ...): ...
def irfft2(x: Any, s: Optional[Any] = ..., axes: Any = ..., norm: str = ..., name: Optional[Any] = ...): ...
def hfft2(x: Any, s: Optional[Any] = ..., axes: Any = ..., norm: str = ..., name: Optional[Any] = ...): ...
def ihfft2(x: Any, s: Optional[Any] = ..., axes: Any = ..., norm: str = ..., name: Optional[Any] = ...): ...
def fftfreq(n: Any, d: float = ..., dtype: Optional[Any] = ..., name: Optional[Any] = ...): ...
def rfftfreq(n: Any, d: float = ..., dtype: Optional[Any] = ..., name: Optional[Any] = ...): ...
def fftshift(x: Any, axes: Optional[Any] = ..., name: Optional[Any] = ...): ...
def ifftshift(x: Any, axes: Optional[Any] = ..., name: Optional[Any] = ...): ...



from __future__ import annotations

from typing import Any, Optional

from paddle.fluid import core as core
from paddle.fluid.framework import Program as Program
from paddle.fluid.framework import default_main_program as default_main_program
from paddle.fluid.framework import default_startup_program as default_startup_program
from paddle.fluid.framework import device_guard as device_guard
from paddle.fluid.framework import name_scope as name_scope
from paddle.optimizer import Optimizer

class LookAhead(Optimizer):
    inner_optimizer: Any = ...
    alpha: Any = ...
    k: Any = ...
    type: str = ...
    helper: Any = ...
    def __init__(self, inner_optimizer: Any, alpha: float = ..., k: int = ..., name: Optional[Any] = ...) -> None: ...
    def step(self) -> None: ...
    def minimize(
        self,
        loss: Any,
        startup_program: Optional[Any] = ...,
        parameters: Optional[Any] = ...,
        no_grad_set: Optional[Any] = ...,
    ): ...



from __future__ import annotations

from collections import defaultdict as defaultdict
from typing import Any

from .process_mesh import ProcessMesh as ProcessMesh

def get_tensor_dist_attr_field_keys(): ...
def get_op_dist_attr_field_keys(): ...
def append_op_input_suffix(name: Any): ...
def append_op_output_suffix(name: Any): ...

class TensorDistributedAttribute:
    def __init__(self) -> None: ...
    @property
    def process_mesh(self): ...
    @process_mesh.setter
    def process_mesh(self, process_mesh: Any) -> None: ...
    @property
    def dims_mapping(self): ...
    @dims_mapping.setter
    def dims_mapping(self, dims_mapping: Any) -> None: ...
    @property
    def shard_sizes(self): ...
    @shard_sizes.setter
    def shard_sizes(self, shard_sizes: Any) -> None: ...
    @property
    def device_placement(self): ...
    @device_placement.setter
    def device_placement(self, device_placement: Any) -> None: ...
    def init(self, dist_attr: Any) -> None: ...
    def is_annotated(self, dist_attr_field_name: Any): ...
    def mark_annotated(self, dist_attr_field_name: Any) -> None: ...
    def mark_annotated_as(self, dist_attr: Any) -> None: ...
    def clear_annotated(self) -> None: ...

class OperatorDistributedAttribute:
    def __init__(self) -> None: ...
    @property
    def process_mesh(self): ...
    @process_mesh.setter
    def process_mesh(self, process_mesh: Any) -> None: ...
    @property
    def op_type(self): ...
    @op_type.setter
    def op_type(self, op_type: Any) -> None: ...
    @property
    def impl_type(self): ...
    @impl_type.setter
    def impl_type(self, impl_type: Any) -> None: ...
    @property
    def impl_idx(self): ...
    @impl_idx.setter
    def impl_idx(self, impl_idx: Any) -> None: ...
    @property
    def is_recompute(self): ...
    @is_recompute.setter
    def is_recompute(self, is_recompute: Any) -> None: ...
    @property
    def inputs_dist_attrs(self): ...
    @property
    def outputs_dist_attrs(self): ...
    def get_input_dist_attr(self, name: Any): ...
    def set_input_dist_attr(self, name: Any, dist_attr: Any) -> None: ...
    def get_output_dist_attr(self, name: Any): ...
    def set_output_dist_attr(self, name: Any, dist_attr: Any) -> None: ...
    def get_input_dims_mapping(self, name: Any): ...
    def set_input_dims_mapping(self, name: Any, dims_mapping: Any) -> None: ...
    def get_output_dims_mapping(self, name: Any): ...
    def set_output_dims_mapping(self, name: Any, dims_mapping: Any) -> None: ...
    def init(self, dist_attr: Any) -> None: ...
    def is_annotated(self, attr_name: Any): ...
    def mark_annotated(self, attr_name: Any) -> None: ...
    def mark_annotated_as(self, dist_attr: Any) -> None: ...
    def clear_annotated(self) -> None: ...
    def is_annotated_input_dims_mapping(self, name: Any): ...
    def is_annotated_output_dims_mapping(self, name: Any): ...



from __future__ import annotations

from typing import Any, Optional

OP_NAME_SCOPE: str
CLIP_OP_NAME_SCOPE: str
STEP_COUNTER: str
OP_ROLE_VAR_ATTR_NAME: Any
RPC_OP_ROLE_ATTR_NAME: Any
RPC_OP_ROLE_ATTR_VALUE: Any
LR_SCHED_OP_ROLE_ATTR_VALUE: Any
OPT_OP_ROLE_ATTR_VALUE: Any
op_role_attr_name: Any
SPARSE_OP_TYPE_DICT: Any
SPARSE_GRAD_OP_TYPE_DICT: Any
DEVICE_LIST: Any
COMMUNICATE_OPS_TYPE: Any
DEFAULT_DEVICE: str

def delete_optimizer_pass(program: Any, config: Any): ...
def distributed_ops_pass(program: Any, config: Any, use_ps_gpu: bool = ...): ...
def append_send_ops_pass(program: Any, config: Any): ...
def init_from_server_pass(program: Any, config: Any): ...
def fake_init_ops_pass(program: Any, config: Any): ...
def ps_gpu_pass(program: Any): ...
def delete_extra_optimizes_pass(program: Any, config: Any): ...
def find_heter_ops(program: Any, default_device: str = ...): ...
def create_heter_program(
    program: Any,
    config: Any,
    heter_program: Any,
    program_block_ops_list: Any,
    heter_ops: Any,
    block_var_detail: Any,
    current_device: Any,
    stage_id: Any,
) -> None: ...
def check_heter_compile_time_strategy(program: Any, config: Any, send_grad_var_list: Any) -> None: ...
def create_trainer_program(
    program: Any, origin_program: Any, config: Any, program_block_ops_list: Any, block_var_detail: Any
) -> None: ...
def insert_communicate_op(
    orign_program: Any,
    config: Any,
    heter_block: Any,
    stage_id: Any,
    first_op_index: Any,
    block_var_detail: Any,
    device: Any,
    is_forward: bool = ...,
): ...
def create_backward_block(program: Any, origin_program: Any, config: Any, bp_ops_list: Any, block_var_detail: Any): ...
def replace_ops_by_communicate_op(
    program: Any, config: Any, heter_block_index: Any, ops_list: Any, block_var_detail: Any
): ...
def remove_trainer_send_op(program: Any, config: Any, heter_block_index: Any, block_var_detail: Any) -> None: ...
def add_heter_send_op(program: Any, heter_program: Any, block: Any, block_var_detail: Any): ...
def find_send_op(program: Any): ...
def get_communicate_var_info(program: Any, block_index: Any, entrance_var_list: Any, type: str = ...): ...
def union_forward_gradient_op(program_block_ops_list: Any): ...
def find_block_joints(program: Any, program_block_ops_list: Any, heter_ops: Any): ...
def find_entrance_exit_private(program: Any, program_block_ops_list: Any): ...
def entrance_exit_check(program: Any, program_block_ops_list: Any, block_var_detail: Any, heter_ops: Any): ...
def find_need_var_from_previous_block(
    need_add_vars: Any, block_var_detail: Any, current_index: Any, heter_ops: Any
): ...
def delete_block_useless_exit(program: Any, program_block_ops_list: Any, block_var_detail: Any): ...
def check_op_device(block: Any, device: Any) -> None: ...
def screen_persistables(program: Any, var_list: Any): ...
def insert_reshape_op(
    program: Any, block: Any, index: Any, var_name: Any, new_var_name: Any, new_var_shape: Optional[Any] = ...
) -> None: ...
def insert_send_concat_op(
    program: Any, block: Any, index: Any, var_name_list: Any, new_var_name: Any, new_var_shape: Any
) -> None: ...
def insert_recv_slice_op(
    program: Any,
    block: Any,
    index: Any,
    var_name: Any,
    var_shape: Any,
    dtype: Any,
    type: Any,
    new_var_name_list: Any,
    new_var_shape_list: Any,
) -> None: ...
def add_heter_trainer_useful_vars(
    config: Any, program: Any, heter_program: Any, heter_block: Any, static_var: Any
) -> None: ...
def delete_trainer_useless_var(config: Any, program: Any, static_var: Any): ...
def block_append_op(program: Any, origin_program: Any, block: Any, op: Any): ...
def add_vars_by_var_list(var_name_list: Any, origin_program: Any, program: Any, block: Any) -> None: ...
def get_varlist_from_op_map(var_map: Any): ...
def find_ops_list_input_output(program: Any, ops_list: Any): ...
def find_op_input_output(program: Any, block: Any, op: Any): ...
def get_vars_name_in_block(block: Any): ...
def is_same_op(op1: Any, op2: Any): ...
def delete_same_ops(block: Any, ops: Any) -> None: ...



from __future__ import annotations

from typing import Any

class Server:
    def __init__(self) -> None: ...

class Worker:
    def __init__(self) -> None: ...

class DownpourServer(Server):
    server_: Any = ...
    def __init__(self) -> None: ...
    def add_sparse_table(self, table_id: Any, learning_rate: Any, slot_key_vars: Any, slot_value_var: Any) -> None: ...
    def add_dense_table(self, table_id: Any, learning_rate: Any, param_var: Any, grad_var: Any): ...
    def get_desc(self): ...

class DownpourWorker(Worker):
    window: Any = ...
    worker_: Any = ...
    def __init__(self, window: Any) -> None: ...
    def add_sparse_table(self, table_id: Any, learning_rate: Any, slot_key_vars: Any, slot_value_vars: Any) -> None: ...
    def add_dense_table(self, table_id: Any, learning_rate: Any, param_vars: Any, grad_vars: Any): ...
    def get_desc(self): ...



from __future__ import annotations

from typing import Any

def seed(seed: Any): ...
def get_cuda_rng_state(): ...
def set_cuda_rng_state(state_list: Any) -> None: ...
def set_random_seed_generator(name: Any, seed: Any) -> None: ...
def get_random_seed_generator(name: Any): ...



from __future__ import annotations

from typing import Any, Optional

class OptimizerWithMixedPrecision:
    def __init__(
        self,
        optimizer: Any,
        amp_lists: Any,
        init_loss_scaling: Any,
        use_dynamic_loss_scaling: Any,
        incr_every_n_steps: Any,
        decr_every_n_nan_or_inf: Any,
        incr_ratio: Any,
        decr_ratio: Any,
        use_pure_fp16: Any,
        use_fp16_guard: Any,
    ) -> None: ...
    def get_loss_scaling(self): ...
    def get_scaled_loss(self): ...
    def backward(
        self,
        loss: Any,
        startup_program: Optional[Any] = ...,
        parameter_list: Optional[Any] = ...,
        no_grad_set: Optional[Any] = ...,
        callbacks: Optional[Any] = ...,
    ): ...
    def amp_init(
        self, place: Any, scope: Optional[Any] = ..., test_program: Optional[Any] = ..., use_fp16_test: bool = ...
    ) -> None: ...
    def apply_gradients(self, params_grads: Any): ...
    def apply_optimize(self, loss: Any, startup_program: Any, params_grads: Any): ...
    def minimize(
        self,
        loss: Any,
        startup_program: Optional[Any] = ...,
        parameter_list: Optional[Any] = ...,
        no_grad_set: Optional[Any] = ...,
    ): ...

def decorate(
    optimizer: Any,
    amp_lists: Optional[Any] = ...,
    init_loss_scaling: Any = ...,
    incr_every_n_steps: int = ...,
    decr_every_n_nan_or_inf: int = ...,
    incr_ratio: float = ...,
    decr_ratio: float = ...,
    use_dynamic_loss_scaling: bool = ...,
    use_pure_fp16: bool = ...,
    use_fp16_guard: Optional[Any] = ...,
): ...



from __future__ import annotations

from typing import Any, Optional

from ..fluid import core as core
from ..fluid import framework as framework
from ..fluid.framework import Variable as Variable
from .adam import Adam as Adam
from .optimizer import Optimizer as Optimizer

class AdamW(Adam):
    type: str = ...
    def __init__(
        self,
        learning_rate: float = ...,
        beta1: float = ...,
        beta2: float = ...,
        epsilon: float = ...,
        parameters: Optional[Any] = ...,
        weight_decay: float = ...,
        lr_ratio: Optional[Any] = ...,
        apply_decay_param_fun: Optional[Any] = ...,
        grad_clip: Optional[Any] = ...,
        lazy_mode: bool = ...,
        multi_precision: bool = ...,
        name: Optional[Any] = ...,
    ) -> None: ...



from __future__ import annotations

from typing import Any, Optional

class FileSystem:
    fs_client: Any = ...
    def __init__(
        self,
        fs_type: str = ...,
        uri: str = ...,
        user: Optional[Any] = ...,
        passwd: Optional[Any] = ...,
        hadoop_bin: str = ...,
    ) -> None: ...
    def get_desc(self): ...

class MPIHelper:
    comm: Any = ...
    MPI: Any = ...
    def __init__(self) -> None: ...
    def get_rank(self): ...
    def get_size(self): ...
    def get_ip(self): ...
    def get_hostname(self): ...
    def finalize(self) -> None: ...



from __future__ import annotations

import unittest
from typing import Any

from imperative_test_utils import fix_model_dict as fix_model_dict

class TestImperativeQatAmp(unittest.TestCase):
    @classmethod
    def setUpClass(cls) -> None: ...
    @classmethod
    def tearDownClass(cls) -> None: ...
    def cache_unzipping(self, target_folder: Any, zip_path: Any) -> None: ...
    def download_model(self, data_url: Any, data_md5: Any, folder_name: Any): ...
    qat: Any = ...
    train_batch_num: int = ...
    train_batch_size: int = ...
    test_batch_num: int = ...
    test_batch_size: int = ...
    eval_acc_top1: float = ...
    def set_vars(self) -> None: ...
    def model_train(self, model: Any, batch_num: int = ..., batch_size: int = ..., use_amp: bool = ...) -> None: ...
    def model_test(self, model: Any, batch_num: int = ..., batch_size: int = ..., use_amp: bool = ...): ...
    def ptq(self) -> None: ...
    def test_ptq(self) -> None: ...



from __future__ import annotations

from typing import Any

from . import ptq_config as ptq_config
from .ptq_registry import PTQRegistry as PTQRegistry

def quant_forward_post_hook(layer: Any, inputs: Any, outputs: Any) -> None: ...



from __future__ import annotations

from typing import Any, Optional

def enabled(): ...
def enable_dygraph(place: Optional[Any] = ...) -> None: ...
def disable_dygraph() -> None: ...
def no_grad(func: Optional[Any] = ...): ...

class no_grad_:
    def __call__(self, func: Any): ...
    orig: Any = ...
    def __enter__(self) -> None: ...
    def __exit__(self, *args: Any) -> None: ...

def guard(place: Optional[Any] = ...) -> None: ...
def grad(
    outputs: Any,
    inputs: Any,
    grad_outputs: Optional[Any] = ...,
    retain_graph: Optional[Any] = ...,
    create_graph: bool = ...,
    only_inputs: bool = ...,
    allow_unused: bool = ...,
    no_grad_vars: Optional[Any] = ...,
): ...
def to_variable(value: Any, name: Optional[Any] = ..., zero_copy: Optional[Any] = ..., dtype: Optional[Any] = ...): ...



from __future__ import annotations

from . import mixed_precision as mixed_precision
from .decoder import *
from .extend_optimizer import *
from .layers import *
from .memory_usage_calc import *
from .mixed_precision import *
from .model_stat import *
from .op_frequence import *
from .optimizer import *
from .quantize import *
from .sparsity import *



from __future__ import annotations

from typing import Any

from paddle.fluid.framework import is_compiled_with_cinn as is_compiled_with_cinn
from paddle.fluid.framework import is_compiled_with_cuda as is_compiled_with_cuda
from paddle.fluid.framework import is_compiled_with_rocm as is_compiled_with_rocm

def is_compiled_with_npu(): ...
def is_compiled_with_ipu(): ...
def IPUPlace(): ...
def is_compiled_with_xpu(): ...
def XPUPlace(dev_id: Any): ...
def is_compiled_with_mlu(): ...
def MLUPlace(dev_id: Any): ...
def get_cudnn_version(): ...
def set_device(device: Any): ...
def get_device(): ...
def get_all_device_type(): ...
def get_all_custom_device_type(): ...
def get_available_device(): ...
def get_available_custom_device(): ...



from __future__ import annotations

from typing import Any, Optional

from ...fluid import dygraph_utils as dygraph_utils
from ...fluid.data_feeder import check_type as check_type
from ...fluid.data_feeder import check_variable_and_dtype as check_variable_and_dtype
from ...fluid.dygraph import BatchNorm as BatchNorm
from ...fluid.dygraph import SpectralNorm as SpectralNorm
from ...framework import ParamAttr as ParamAttr
from ...framework import get_default_dtype as get_default_dtype
from ...framework import no_grad as no_grad
from ...framework import set_default_dtype as set_default_dtype
from .. import Layer as Layer
from ..functional import batch_norm as batch_norm
from ..functional import instance_norm as instance_norm
from ..functional import layer_norm as layer_norm
from ..initializer import Constant as Constant

class _InstanceNormBase(Layer):
    scale: Any = ...
    bias: Any = ...
    def __init__(
        self,
        num_features: Any,
        epsilon: float = ...,
        momentum: float = ...,
        weight_attr: Optional[Any] = ...,
        bias_attr: Optional[Any] = ...,
        data_format: str = ...,
        name: Optional[Any] = ...,
    ) -> None: ...
    def forward(self, input: Any): ...
    def extra_repr(self): ...

class InstanceNorm1D(_InstanceNormBase): ...
class InstanceNorm2D(_InstanceNormBase): ...
class InstanceNorm3D(_InstanceNormBase): ...

class GroupNorm(Layer):
    weight: Any = ...
    bias: Any = ...
    def __init__(
        self,
        num_groups: Any,
        num_channels: Any,
        epsilon: float = ...,
        weight_attr: Optional[Any] = ...,
        bias_attr: Optional[Any] = ...,
        data_format: str = ...,
        name: Optional[Any] = ...,
    ) -> None: ...
    def forward(self, input: Any): ...
    def extra_repr(self): ...

class LayerNorm(Layer):
    weight: Any = ...
    bias: Any = ...
    def __init__(
        self,
        normalized_shape: Any,
        epsilon: float = ...,
        weight_attr: Optional[Any] = ...,
        bias_attr: Optional[Any] = ...,
        name: Optional[Any] = ...,
    ) -> None: ...
    def forward(self, input: Any): ...
    def extra_repr(self): ...

class _BatchNormBase(Layer):
    weight: Any = ...
    bias: Any = ...
    def __init__(
        self,
        num_features: Any,
        momentum: float = ...,
        epsilon: float = ...,
        weight_attr: Optional[Any] = ...,
        bias_attr: Optional[Any] = ...,
        data_format: str = ...,
        use_global_stats: Optional[Any] = ...,
        name: Optional[Any] = ...,
    ) -> None: ...
    def forward(self, input: Any): ...
    def extra_repr(self): ...

class BatchNorm1D(_BatchNormBase):
    def __init__(
        self,
        num_features: Any,
        momentum: float = ...,
        epsilon: float = ...,
        weight_attr: Optional[Any] = ...,
        bias_attr: Optional[Any] = ...,
        data_format: str = ...,
        use_global_stats: Optional[Any] = ...,
        name: Optional[Any] = ...,
    ) -> None: ...

class BatchNorm2D(_BatchNormBase): ...

class BatchNorm3D(_BatchNormBase):
    def __init__(
        self,
        num_features: Any,
        momentum: float = ...,
        epsilon: float = ...,
        weight_attr: Optional[Any] = ...,
        bias_attr: Optional[Any] = ...,
        data_format: str = ...,
        use_global_stats: Optional[Any] = ...,
        name: Optional[Any] = ...,
    ) -> None: ...

class SyncBatchNorm(_BatchNormBase):
    def __init__(
        self,
        num_features: Any,
        momentum: float = ...,
        epsilon: float = ...,
        weight_attr: Optional[Any] = ...,
        bias_attr: Optional[Any] = ...,
        data_format: str = ...,
        name: Optional[Any] = ...,
    ) -> None: ...
    def forward(self, x: Any): ...
    @classmethod
    def convert_sync_batchnorm(cls, layer: Any): ...

class LocalResponseNorm(Layer):
    size: Any = ...
    alpha: Any = ...
    beta: Any = ...
    k: Any = ...
    data_format: Any = ...
    name: Any = ...
    def __init__(
        self,
        size: Any,
        alpha: float = ...,
        beta: float = ...,
        k: float = ...,
        data_format: str = ...,
        name: Optional[Any] = ...,
    ) -> None: ...
    def forward(self, input: Any): ...
    def extra_repr(self): ...



from __future__ import annotations

from typing import Any

class Storable:
    def get_state(self) -> None: ...
    def set_state(self, state: Any) -> None: ...
    def save(self, path: Any): ...
    def load(self, path: Any) -> None: ...



from __future__ import annotations

from typing import Any, Optional

from google.protobuf import text_format as text_format

from ..backward import append_backward as append_backward
from .node import DownpourServer as DownpourServer
from .node import DownpourWorker as DownpourWorker

class DownpourSGD:
    learning_rate_: Any = ...
    window_: Any = ...
    type: str = ...
    data_norm_name: Any = ...
    def __init__(self, learning_rate: float = ..., window: int = ...) -> None: ...
    def minimize(
        self,
        losses: Any,
        startup_program: Optional[Any] = ...,
        parameter_list: Optional[Any] = ...,
        no_grad_set: Optional[Any] = ...,
    ): ...



from __future__ import annotations

from .math import segment_max as segment_max
from .math import segment_mean as segment_mean
from .math import segment_min as segment_min
from .math import segment_sum as segment_sum



from __future__ import annotations

from typing import Any, Optional

class Role:
    WORKER: int = ...
    SERVER: int = ...
    XPU: int = ...

class MockBarrier:
    def barrier(self) -> None: ...
    def barrier_all(self) -> None: ...
    def all_reduce(self, obj: Any): ...
    def all_gather(self, obj: Any): ...

class RoleMakerBase:
    def __init__(self) -> None: ...
    def is_worker(self) -> None: ...
    def is_server(self) -> None: ...
    def is_first_worker(self) -> None: ...
    def worker_num(self) -> None: ...
    def role_id(self): ...
    def worker_index(self) -> None: ...
    def server_index(self) -> None: ...
    def get_trainer_endpoints(self): ...
    def get_pserver_endpoints(self): ...
    def to_string(self): ...
    def all_gather(self, input: Any) -> None: ...
    def all_reduce_worker(self, input: Any, output: Any, mode: str = ...) -> None: ...
    def barrier_worker(self) -> None: ...
    def barrier_all(self) -> None: ...

class MPIRoleMaker(RoleMakerBase):
    MPI: Any = ...
    def __init__(self) -> None: ...
    def get_local_ip(self): ...
    def generate_role(self) -> None: ...

class MPISymetricRoleMaker(MPIRoleMaker):
    def __init__(self) -> None: ...
    def all_gather(self, input: Any): ...
    def all_reduce_worker(self, input: Any, output: Any, mode: str = ...) -> None: ...
    def barrier_worker(self) -> None: ...
    def barrier_all(self) -> None: ...
    def is_first_worker(self): ...
    def get_pserver_endpoints(self): ...
    def worker_num(self): ...
    def is_worker(self): ...
    def is_server(self): ...
    def worker_index(self): ...
    def server_index(self): ...
    def generate_role(self) -> None: ...

class PaddleCloudRoleMaker(RoleMakerBase):
    def __init__(self, is_collective: bool = ...) -> None: ...
    def generate_role(self) -> None: ...
    def get_pserver_endpoints(self): ...
    def is_worker(self): ...
    def is_server(self): ...
    def is_first_worker(self): ...
    def worker_index(self): ...
    def server_index(self): ...
    def worker_num(self): ...

class GeneralRoleMaker(RoleMakerBase):
    def __init__(self, **kwargs: Any) -> None: ...
    def generate_role(self) -> None: ...
    def all_gather(self, input: Any): ...
    def all_reduce_worker(self, input: Any, output: Any, mode: str = ...) -> None: ...
    def barrier_worker(self) -> None: ...
    def barrier_all(self) -> None: ...
    def get_local_endpoint(self): ...
    def get_trainer_endpoints(self): ...
    def get_pserver_endpoints(self): ...
    def is_worker(self): ...
    def is_server(self): ...
    def is_first_worker(self): ...
    def worker_index(self): ...
    def server_index(self): ...
    def worker_num(self): ...
    def server_num(self): ...

class HeterRoleMaker(GeneralRoleMaker):
    def generate_role(self) -> None: ...
    def is_xpu(self): ...
    def is_first_xpu(self): ...
    def xpu_num(self): ...

class UserDefinedRoleMaker(RoleMakerBase):
    def __init__(
        self, current_id: int = ..., role: Any = ..., worker_num: int = ..., server_endpoints: Optional[Any] = ...
    ) -> None: ...
    def generate_role(self) -> None: ...
    def is_worker(self): ...
    def is_server(self): ...
    def is_first_worker(self): ...
    def worker_index(self): ...
    def server_index(self): ...
    def worker_num(self): ...

class UserDefinedCollectiveRoleMaker(RoleMakerBase):
    def __init__(self, current_id: int = ..., worker_endpoints: Optional[Any] = ...) -> None: ...
    def generate_role(self) -> None: ...
    def is_worker(self): ...
    def is_first_worker(self): ...
    def worker_index(self): ...
    def worker_num(self): ...



from __future__ import annotations

from typing import Any

class Constraint:
    def __call__(self, value: Any) -> None: ...

class Real(Constraint):
    def __call__(self, value: Any): ...

class Range(Constraint):
    def __init__(self, lower: Any, upper: Any) -> None: ...
    def __call__(self, value: Any): ...

class Positive(Constraint):
    def __call__(self, value: Any): ...

class Simplex(Constraint):
    def __call__(self, value: Any): ...

real: Any
positive: Any
simplex: Any



from __future__ import annotations

from typing import Any, Optional

from ..layer.common import Linear as Linear
from ..layer.conv import Conv1DTranspose as Conv1DTranspose
from ..layer.conv import Conv2DTranspose as Conv2DTranspose
from ..layer.conv import Conv3DTranspose as Conv3DTranspose

def normal_(x: Any, mean: float = ..., std: float = ...): ...

class SpectralNorm:
    name: Any = ...
    dim: Any = ...
    n_power_iterations: Any = ...
    eps: Any = ...
    def __init__(self, name: str = ..., n_power_iterations: int = ..., dim: int = ..., eps: float = ...) -> None: ...
    def reshape_weight_to_matrix(self, weight: Any): ...
    def compute_weight(self, layer: Any, do_power_iteration: Any): ...
    def __call__(self, layer: Any, inputs: Any) -> None: ...
    @staticmethod
    def apply(layer: Any, name: Any, n_power_iterations: Any, dim: Any, eps: Any): ...

def spectral_norm(
    layer: Any, name: str = ..., n_power_iterations: int = ..., eps: float = ..., dim: Optional[Any] = ...
): ...



from __future__ import annotations

from .fs import HDFSClient as HDFSClient
from .fs import LocalFS as LocalFS
from .ps_util import DistributedInfer as DistributedInfer
from .recompute import recompute as recompute



from __future__ import annotations

from paddle.distribution.beta import Beta as Beta
from paddle.distribution.categorical import Categorical as Categorical
from paddle.distribution.dirichlet import Dirichlet as Dirichlet
from paddle.distribution.distribution import Distribution as Distribution
from paddle.distribution.exponential_family import (
    ExponentialFamily as ExponentialFamily,
)
from paddle.distribution.independent import Independent as Independent
from paddle.distribution.kl import kl_divergence as kl_divergence
from paddle.distribution.kl import register_kl as register_kl
from paddle.distribution.multinomial import Multinomial as Multinomial
from paddle.distribution.normal import Normal as Normal
from paddle.distribution.transform import *
from paddle.distribution.transformed_distribution import (
    TransformedDistribution as TransformedDistribution,
)
from paddle.distribution.uniform import Uniform as Uniform



from __future__ import annotations

from typing import Any

from paddle.fluid.dygraph.layers import Layer as Layer

from ..utils.hybrid_parallel_util import (
    broadcast_sharding_parameters as broadcast_sharding_parameters,
)
from ..utils.log_util import logger as logger
from .meta_parallel_base import MetaParallelBase as MetaParallelBase

class ShardingParallel(MetaParallelBase):
    def __init__(self, layers: Any, hcg: Any, **kwargs: Any) -> None: ...



from __future__ import annotations

from .convert_call_func import convert_call as convert_call
from .convert_operators import cast_bool_if_necessary as cast_bool_if_necessary
from .convert_operators import choose_shape_attr_or_api as choose_shape_attr_or_api
from .convert_operators import convert_assert as convert_assert
from .convert_operators import convert_ifelse as convert_ifelse
from .convert_operators import convert_len as convert_len
from .convert_operators import convert_logical_and as convert_logical_and
from .convert_operators import convert_logical_not as convert_logical_not
from .convert_operators import convert_logical_or as convert_logical_or
from .convert_operators import convert_pop as convert_pop
from .convert_operators import convert_print as convert_print
from .convert_operators import convert_shape_compare as convert_shape_compare
from .convert_operators import convert_var_dtype as convert_var_dtype
from .convert_operators import convert_var_shape as convert_var_shape
from .convert_operators import convert_var_shape_simple as convert_var_shape_simple
from .convert_operators import convert_while_loop as convert_while_loop
from .convert_operators import eval_if_exist_else_none as eval_if_exist_else_none
from .variable_trans_func import create_bool_as_type as create_bool_as_type
from .variable_trans_func import create_fill_constant_node as create_fill_constant_node
from .variable_trans_func import (
    create_static_variable_gast_node as create_static_variable_gast_node,
)
from .variable_trans_func import data_layer_not_check as data_layer_not_check
from .variable_trans_func import to_static_variable as to_static_variable
from .variable_trans_func import (
    to_static_variable_gast_node as to_static_variable_gast_node,
)



from __future__ import annotations

from typing import Any, Optional

from ...fluid.data_feeder import check_dtype as check_dtype
from ...fluid.data_feeder import check_variable_and_dtype as check_variable_and_dtype
from ...fluid.framework import in_dygraph_mode as in_dygraph_mode
from ...fluid.layer_helper import LayerHelper as LayerHelper
from ...static import Variable as Variable

def one_hot(x: Any, num_classes: Any, name: Optional[Any] = ...): ...
def embedding(x: Any, weight: Any, padding_idx: Optional[Any] = ..., sparse: bool = ..., name: Optional[Any] = ...): ...



from __future__ import annotations

from .amp import *
from .base import *
from .checkpoint import *
from .container import *
from .dygraph_to_static import ProgramTranslator as ProgramTranslator
from .io import *
from .jit import *
from .layers import *
from .learning_rate_scheduler import *
from .nn import *
from .parallel import *
from .rnn import *
from .tracer import *



from __future__ import annotations

from typing import Any, Optional

from paddle import nn

from ...meta_optimizers.dygraph_optimizer.sharding_optimizer_stage2 import (
    ShardingOptimizerStage2 as ShardingOptimizerStage2,
)
from ...utils.internal_storage import GradStorage as GradStorage
from .sharding_utils import Taskflow as Taskflow
from .sharding_utils import Type as Type

class ShardingStage2(nn.Layer):
    def __init__(
        self,
        layer: Any,
        sharding_optimizer: Any,
        group: Optional[Any] = ...,
        sync_buffers: bool = ...,
        buffer_max_size: Any = ...,
        auto_refresh_trainable: bool = ...,
        device: str = ...,
    ): ...
    def forward(self, *inputs: Any, **kwargs: Any): ...
    def set_state_dict(self, state_dict: Any, use_structured_name: bool = ...) -> None: ...
    def state_dict(
        self, destination: Optional[Any] = ..., include_sublayers: bool = ..., structured_name_prefix: str = ...
    ): ...
    def to(self, device: Optional[Any] = ..., dtype: Optional[Any] = ..., blocking: bool = ...) -> None: ...
    def __getattr__(self, name: Any): ...



from __future__ import annotations

from typing import Any

class DataGenerator:
    batch_size_: int = ...
    def __init__(self) -> None: ...
    def set_batch(self, batch_size: Any) -> None: ...
    def run_from_memory(self) -> None: ...
    def run_from_stdin(self) -> None: ...
    def generate_sample(self, line: Any) -> None: ...
    def generate_batch(self, samples: Any): ...

class MultiSlotStringDataGenerator(DataGenerator): ...
class MultiSlotDataGenerator(DataGenerator): ...



from __future__ import annotations

from enum import Enum
from typing import Any, Optional

SUCC: int
PRED: int

class CostNodeType(Enum):
    DEFAULT = ...
    COMPUTATION = ...
    COMMUNICATION = ...
    VARIABLE = ...
    MERGED = ...
    NOP = ...

class Cost:
    runtime: Any = ...
    static_mem: Any = ...
    peak_mem: Any = ...
    def __init__(self) -> None: ...

class CostModelMode(Enum):
    DEFAULT = ...
    BENCHMARKING = ...
    ANALYSIS = ...
    MIXED = ...

class CostNode:
    id: Any = ...
    node: Any = ...
    type: Any = ...
    is_optim: bool = ...
    is_bwd: bool = ...
    def __init__(self, node: Any, node_type: Any, id: Optional[Any] = ...) -> None: ...
    @property
    def cost(self): ...
    @cost.setter
    def cost(self, cost: Any) -> None: ...

class MergedOpsCostNode(CostNode):
    node_list: Any = ...
    is_bwd: Any = ...
    def __init__(
        self, node_type: Any, id: Optional[Any] = ..., base_node_list: Optional[Any] = ..., is_bwd: bool = ...
    ) -> None: ...

class CommOpCostNode(CostNode):
    node_list: Any = ...
    ranks: Any = ...
    comm_type: Any = ...
    is_bwd: Any = ...
    def __init__(
        self,
        node: Any,
        node_type: Any,
        id: Optional[Any] = ...,
        comm_node_list: Optional[Any] = ...,
        is_bwd: bool = ...,
    ) -> None: ...
    def set_ranks(self, ranks: Any) -> None: ...
    input_shape: Any = ...
    output_shape: Any = ...
    def set_shapes(self, input_shape: Any, output_shape: Any) -> None: ...
    def init_comm_cost(self, cluster: Optional[Any] = ...) -> None: ...

class TensorCostNode(CostNode):
    shape: Any = ...
    dtype: Any = ...
    dtype_factor: int = ...
    persistable: Any = ...
    shared_node_id: Any = ...
    batch_size: Any = ...
    def __init__(
        self,
        node: Any,
        node_type: Any,
        id: Optional[Any] = ...,
        base_node_list: Optional[Any] = ...,
        batch_size: Optional[Any] = ...,
        shared_node_id: Optional[Any] = ...,
    ) -> None: ...
    def get_size(self): ...

class CompOpCostNode(CostNode):
    is_bwd: Any = ...
    is_optim: Any = ...
    def __init__(
        self, node: Any, node_type: Any, id: Optional[Any] = ..., is_bwd: bool = ..., is_optim: bool = ...
    ) -> None: ...
    cost: Any = ...
    def init_comp_cost(self, cost_data: Any) -> None: ...

class PipeEvent:
    stage_id: Any = ...
    name: Any = ...
    duration: Any = ...
    s_time: Any = ...
    e_time: int = ...
    def __init__(self, stage_id: Any, event_name: Any, duration: Any, start_time: int = ...) -> None: ...

class CostModel:
    mode: Any = ...
    opcall_overhead: Any = ...
    batch_size: Any = ...
    microbatch_num: Any = ...
    nodes: Any = ...
    origin_graph: Any = ...
    op_graph: Any = ...
    runtime_graph: Any = ...
    cluster: Any = ...
    cost_data: Any = ...
    pp2rank: Any = ...
    rank2pp: Any = ...
    ring2rank: Any = ...
    fwd_time: Any = ...
    bwd_time: Any = ...
    optim_time: Any = ...
    def __init__(
        self,
        mode: Any = ...,
        cluster: Optional[Any] = ...,
        batch_size: int = ...,
        microbatch_num: int = ...,
        opcall_overhead: int = ...,
        standalone_cost_data: Optional[Any] = ...,
        pipeline_config: Optional[Any] = ...,
    ) -> None: ...
    distributed_program: Any = ...
    total_rank: Any = ...
    def parse_program(self, distributed_program: Any): ...
    def build_op_graph(self) -> None: ...
    def build_runtime_graph(self) -> None: ...
    def eliminate_multi_edges(self, graph: Optional[Any] = ...) -> None: ...
    def merge_comm(self) -> None: ...
    def merge_linear(self): ...
    def merge_branch(self): ...
    def get_runtime_cost(self): ...
    def get_mem(self): ...
    def get_pipeline_time(self): ...
    def get_cost(self): ...
    def init(self, distributed_program: Any) -> None: ...

def estimate_cost(
    distributed_program: Any, cluster: Any, pipeline_config: Any, standalone_cost_data: Any, batch_size: Any
): ...



from __future__ import annotations

from .collective_runtime import CollectiveRuntime as CollectiveRuntime
from .parameter_server_runtime import ParameterServerRuntime as ParameterServerRuntime
from .the_one_ps import TheOnePSRuntime as TheOnePSRuntime



from __future__ import annotations

from typing import Any, Optional

from paddle.fluid.dygraph import Layer

class BasicGRUUnit(Layer):
    def __init__(
        self,
        name_scope: Any,
        hidden_size: Any,
        param_attr: Optional[Any] = ...,
        bias_attr: Optional[Any] = ...,
        gate_activation: Optional[Any] = ...,
        activation: Optional[Any] = ...,
        dtype: str = ...,
    ) -> None: ...
    def forward(self, input: Any, pre_hidden: Any): ...

def basic_gru(
    input: Any,
    init_hidden: Any,
    hidden_size: Any,
    num_layers: int = ...,
    sequence_length: Optional[Any] = ...,
    dropout_prob: float = ...,
    bidirectional: bool = ...,
    batch_first: bool = ...,
    param_attr: Optional[Any] = ...,
    bias_attr: Optional[Any] = ...,
    gate_activation: Optional[Any] = ...,
    activation: Optional[Any] = ...,
    dtype: str = ...,
    name: str = ...,
): ...
def basic_lstm(
    input: Any,
    init_hidden: Any,
    init_cell: Any,
    hidden_size: Any,
    num_layers: int = ...,
    sequence_length: Optional[Any] = ...,
    dropout_prob: float = ...,
    bidirectional: bool = ...,
    batch_first: bool = ...,
    param_attr: Optional[Any] = ...,
    bias_attr: Optional[Any] = ...,
    gate_activation: Optional[Any] = ...,
    activation: Optional[Any] = ...,
    forget_bias: float = ...,
    dtype: str = ...,
    name: str = ...,
): ...

class BasicLSTMUnit(Layer):
    def __init__(
        self,
        name_scope: Any,
        hidden_size: Any,
        param_attr: Optional[Any] = ...,
        bias_attr: Optional[Any] = ...,
        gate_activation: Optional[Any] = ...,
        activation: Optional[Any] = ...,
        forget_bias: float = ...,
        dtype: str = ...,
    ) -> None: ...
    def forward(self, input: Any, pre_hidden: Any, pre_cell: Any): ...



from __future__ import annotations

from typing import Any, Optional

from google.protobuf import text_format as text_format

from . import core as core
from . import io as io
from . import unique_name as unique_name
from .framework import Program as Program
from .framework import Variable as Variable
from .framework import default_main_program as default_main_program
from .graphviz import GraphPreviewGenerator as GraphPreviewGenerator
from .layer_helper import LayerHelper as LayerHelper
from .proto import framework_pb2 as framework_pb2

def repr_data_type(type: Any): ...
def repr_tensor(proto: Any): ...

reprtpl: str

def repr_lodtensor(proto: Any): ...
def repr_selected_rows(proto: Any): ...
def repr_tensor_array(proto: Any): ...

type_handlers: Any

def repr_var(vardesc: Any): ...
def pprint_program_codes(program_desc: Any): ...
def pprint_block_codes(block_desc: Any, show_backward: bool = ...): ...
def repr_attr(desc: Any): ...

op_repr_handlers: Any

def repr_op(opdesc: Any): ...
def draw_block_graphviz(block: Any, highlights: Optional[Any] = ..., path: str = ...): ...



from __future__ import annotations

from .asp import decorate as decorate
from .asp import prune_model as prune_model
from .asp import reset_excluded_layers as reset_excluded_layers
from .asp import set_excluded_layers as set_excluded_layers
from .supported_layer_list import add_supported_layer as add_supported_layer
from .utils import CheckMethod as CheckMethod
from .utils import MaskAlgo as MaskAlgo
from .utils import calculate_density as calculate_density
from .utils import check_mask_1d as check_mask_1d
from .utils import check_mask_2d as check_mask_2d
from .utils import check_sparsity as check_sparsity
from .utils import create_mask as create_mask
from .utils import get_mask_1d as get_mask_1d
from .utils import get_mask_2d_best as get_mask_2d_best
from .utils import get_mask_2d_greedy as get_mask_2d_greedy



from __future__ import annotations

from typing import Any

def noam_decay(d_model: Any, warmup_steps: Any, learning_rate: float = ...): ...
def exponential_decay(learning_rate: Any, decay_steps: Any, decay_rate: Any, staircase: bool = ...): ...
def natural_exp_decay(learning_rate: Any, decay_steps: Any, decay_rate: Any, staircase: bool = ...): ...
def inverse_time_decay(learning_rate: Any, decay_steps: Any, decay_rate: Any, staircase: bool = ...): ...
def polynomial_decay(
    learning_rate: Any, decay_steps: Any, end_learning_rate: float = ..., power: float = ..., cycle: bool = ...
): ...
def piecewise_decay(boundaries: Any, values: Any): ...
def cosine_decay(learning_rate: Any, step_each_epoch: Any, epochs: Any): ...
def linear_lr_warmup(learning_rate: Any, warmup_steps: Any, start_lr: Any, end_lr: Any): ...



from __future__ import annotations

from ..fluid.clip import ClipGradByGlobalNorm as ClipGradByGlobalNorm
from ..fluid.clip import ClipGradByNorm as ClipGradByNorm
from ..fluid.clip import ClipGradByValue as ClipGradByValue



from __future__ import annotations

from enum import Enum
from typing import Any, Callable, Iterable, Optional, Union

from .profiler_statistic import SortedKeys as SortedKeys
from .profiler_statistic import StatisticData as StatisticData
from .timer import benchmark as benchmark
from .utils import RecordEvent as RecordEvent
from .utils import wrap_optimizers as wrap_optimizers

class ProfilerState(Enum):
    CLOSED = ...
    READY = ...
    RECORD = ...
    RECORD_AND_RETURN = ...

class ProfilerTarget(Enum):
    CPU = ...
    GPU = ...
    MLU = ...

def make_scheduler(closed: int, ready: int, record: int, *, repeat: int = ..., skip_first: int = ...) -> Callable: ...
def export_chrome_tracing(dir_name: str, worker_name: Optional[str] = ...) -> Callable: ...
def export_protobuf(dir_name: str, worker_name: Optional[str] = ...) -> Callable: ...

class Profiler:
    targets: Any = ...
    profiler: Any = ...
    scheduler: Any = ...
    on_trace_ready: Any = ...
    step_num: int = ...
    previous_state: Any = ...
    current_state: Any = ...
    record_event: Any = ...
    profiler_result: Any = ...
    timer_only: Any = ...
    def __init__(
        self,
        *,
        targets: Optional[Iterable[ProfilerTarget]] = ...,
        scheduler: Union[Callable[[int], ProfilerState], tuple, None] = ...,
        on_trace_ready: Optional[Callable[..., Any]] = ...,
        timer_only: Optional[bool] = ...,
    ) -> None: ...
    def __enter__(self): ...
    def __exit__(self, exc_type: Any, exc_val: Any, exc_tb: Any) -> None: ...
    def start(self) -> None: ...
    def stop(self) -> None: ...
    def step(self, num_samples: Optional[int] = ...) -> Any: ...
    def step_info(self, unit: Optional[Any] = ...): ...
    def export(self, path: str = ..., format: str = ...) -> None: ...
    def summary(
        self, sorted_by: Any = ..., op_detail: bool = ..., thread_sep: bool = ..., time_unit: str = ...
    ) -> None: ...

def get_profiler(config_path: Any): ...



from __future__ import annotations

from typing import Any

from paddle.fluid.dygraph import layers

class _ProgramHolder:
    def __init__(self, program_desc: Any) -> None: ...
    @property
    def infer_program(self): ...
    @property
    def train_program(self): ...
    @property
    def input_descs(self): ...
    @property
    def output_descs(self): ...
    @property
    def persistable_names(self): ...
    @property
    def double_grad_descs(self): ...
    @property
    def scope(self): ...

class TranslatedLayer(layers.Layer):
    def __init__(self, programs: Any, persistable_vars: Any) -> None: ...
    training: bool = ...
    def train(self) -> None: ...
    def eval(self) -> None: ...
    def program(self, method_name: str = ...): ...



from __future__ import annotations

from .creation import sparse_coo_tensor as sparse_coo_tensor
from .creation import sparse_csr_tensor as sparse_csr_tensor
from .layer.activation import ReLU as ReLU



from __future__ import annotations

import unittest

def conv_block(): ...

class TestGraph(unittest.TestCase):
    def graph_apis(self, use_cuda: bool = ..., for_ci: bool = ...) -> None: ...
    def test_graph_apis_cpu(self) -> None: ...
    def test_graph_apis_cuda(self) -> None: ...



from __future__ import annotations

from .parallel_layers import ColumnParallelLinear as ColumnParallelLinear
from .parallel_layers import LayerDesc as LayerDesc
from .parallel_layers import ParallelCrossEntropy as ParallelCrossEntropy
from .parallel_layers import PipelineLayer as PipelineLayer
from .parallel_layers import RNGStatesTracker as RNGStatesTracker
from .parallel_layers import RowParallelLinear as RowParallelLinear
from .parallel_layers import SharedLayerDesc as SharedLayerDesc
from .parallel_layers import VocabParallelEmbedding as VocabParallelEmbedding
from .parallel_layers import get_rng_state_tracker as get_rng_state_tracker
from .parallel_layers import model_parallel_random_seed as model_parallel_random_seed
from .pipeline_parallel import PipelineParallel as PipelineParallel
from .sharding_parallel import ShardingParallel as ShardingParallel
from .tensor_parallel import TensorParallel as TensorParallel



from __future__ import annotations

from typing import Callable

from typing_extensions import ParamSpec, TypeVar

InputArgs = ParamSpec("InputArgs")
RetValue = TypeVar("RetValue")

from .._typing import CPUPlace as CPUPlace
from .._typing import CUDAPinnedPlace as CUDAPinnedPlace
from .._typing import CUDAPlace as CUDAPlace
from .._typing import CustomPlace as CustomPlace
from .._typing import IPUPlace as IPUPlace
from .._typing import MLUPlace as MLUPlace
from .._typing import NPUPlace as NPUPlace
from ..fluid import core as core
from ..fluid import monkey_patch_variable as monkey_patch_variable
from ..fluid.core import VarBase as VarBase
from ..fluid.dygraph import monkey_patch_math_varbase as monkey_patch_math_varbase
from ..fluid.dygraph.base import grad as grad
from ..fluid.dygraph.parallel import DataParallel as DataParallel
from ..fluid.framework import OpProtoHolder as OpProtoHolder
from ..fluid.framework import convert_np_dtype_to_dtype_ as convert_np_dtype_to_dtype_
from ..fluid.framework import disable_signal_handler as disable_signal_handler
from ..fluid.framework import dygraph_only as dygraph_only
from ..fluid.framework import get_flags as get_flags
from ..fluid.framework import set_flags as set_flags
from ..fluid.layers.tensor import create_parameter as create_parameter
from ..fluid.param_attr import ParamAttr as ParamAttr
from . import random as random
from .framework import get_default_dtype as get_default_dtype
from .framework import is_grad_enabled as is_grad_enabled
from .framework import set_default_dtype as set_default_dtype
from .framework import set_grad_enabled as set_grad_enabled
from .io import load as load
from .io import save as save
from .random import seed as seed

def no_grad(func: Callable[InputArgs, RetValue]) -> Callable[InputArgs, RetValue]: ...
def in_dynamic_mode() -> bool: ...



from __future__ import annotations

from typing import Any, Optional

def one_hot(input: Any, depth: Any, allow_out_of_range: bool = ...): ...
def embedding(
    input: Any,
    size: Any,
    is_sparse: bool = ...,
    is_distributed: bool = ...,
    padding_idx: Optional[Any] = ...,
    param_attr: Optional[Any] = ...,
    dtype: str = ...,
): ...



from __future__ import annotations

from enum import Enum
from typing import Any

class MaskAlgo(Enum):
    MASK_1D = ...
    MASK_2D_GREEDY = ...
    MASK_2D_BEST = ...

class CheckMethod(Enum):
    CHECK_1D = ...
    CHECK_2D = ...
    @staticmethod
    def get_checking_method(mask_algo: Any): ...

def calculate_density(x: Any): ...
def check_mask_1d(mat: Any, n: Any, m: Any): ...
def get_mask_1d(mat: Any, n: Any, m: Any): ...
def check_mask_2d(mat: Any, n: Any, m: Any): ...
def get_mask_2d_greedy(mat: Any, n: Any, m: Any): ...
def get_mask_2d_best(mat: Any, n: Any, m: Any): ...
def create_mask(tensor: Any, func_name: Any = ..., n: int = ..., m: int = ...): ...
def check_sparsity(tensor: Any, func_name: Any = ..., n: int = ..., m: int = ...): ...



from __future__ import annotations

from typing import Any

URL_DEV_TEST: str
MD5_DEV_TEST: str
URL_TRAIN: str
MD5_TRAIN: str
URL_MODEL: str
MD5_MODEL: str
START: str
END: str
UNK: str
UNK_IDX: int

def reader_creator(tar_file: Any, file_name: Any, dict_size: Any): ...
def train(dict_size: Any): ...
def test(dict_size: Any): ...
def gen(dict_size: Any): ...
def get_dict(dict_size: Any, reverse: bool = ...): ...
def fetch() -> None: ...



from __future__ import annotations

from typing import Any

from paddle.fluid.framework import static_only as static_only

from ..utils import get_logger as get_logger
from .converter import Converter as Converter
from .utils import get_dist_attr as get_dist_attr

def check_filename(re_exp: Any, filename: Any): ...

class DistributedSaver:
    def __init__(self) -> None: ...
    def save(self, path: Any, serial_program: Any, dist_main_program: Any, dist_context: Any) -> None: ...
    def load(
        self, path: Any, program: Any, dist_context: Any, strict: bool = ..., load_optimizer: bool = ...
    ) -> None: ...
    def save_inference_model(self, path: Any, feed_vars: Any, fetch_vars: Any, exe: Any, **kwargs: Any): ...



from __future__ import annotations

from typing import Any

class ProcessMesh:
    def __init__(self, mesh: Any) -> None: ...
    @property
    def topology(self): ...
    @property
    def processes(self): ...
    @property
    def ndim(self): ...
    def __eq__(self, other: Any) -> Any: ...
    def __ne__(self, other: Any) -> Any: ...



from __future__ import annotations

from typing import Any, Optional

from ..meta_parallel.sharding.sharding_utils import Type as Type
from ..meta_parallel.sharding.sharding_utils import device_guard as device_guard

class InternalStorage:
    buffer: Any = ...
    def __init__(self, size: Any, dtype: Any, device: Any, convert_cpu: bool = ...) -> None: ...
    def to(self, device: Any, dtype: Optional[Any] = ..., keep_alignment: bool = ...) -> None: ...

class ParamStorage(InternalStorage):
    param2align: Any = ...
    def __init__(self, size: Any, dtype: Any, device: Any) -> None: ...
    def to(self, device: Any, dtype: Optional[Any] = ..., keep_alignment: bool = ...) -> None: ...
    buffer: Any = ...
    def add_rank_params(self, trainable_params: Any, param2align: Any, convert_gpu: bool = ...) -> None: ...

class GradStorage(InternalStorage):
    params_checked_in: int = ...
    destination: Any = ...
    sent: bool = ...
    def __init__(
        self, size: Any, dtype: Any, device: Any, destination: Any, parm2align: Any, convert_cpu: bool = ...
    ) -> None: ...
    def reset_checked_in(self) -> None: ...
    @property
    def all_checked_in(self): ...
    def can_add_grad_view(self, param: Any, align: Any): ...
    def to(self, device: Any, dtype: Optional[Any] = ..., keep_alignment: bool = ...) -> None: ...
    def add_grad(self, param: Any, align: Any) -> None: ...
    buffer: Any = ...
    def manumal_relase(self) -> None: ...
    def rebuild(self) -> None: ...



from __future__ import annotations

from typing import Any, Optional

from paddle.fluid.dygraph import Layer as Layer
from paddle.fluid.dygraph import LayerList as LayerList

from ..base.private_helper_function import wait_server_ready as wait_server_ready
from .common import OP_ROLE_KEY as OP_ROLE_KEY
from .common import OP_ROLE_VAR_KEY as OP_ROLE_VAR_KEY
from .common import CollectiveHelper as CollectiveHelper
from .common import OpRole as OpRole
from .common import is_backward_op as is_backward_op
from .common import is_loss_grad_op as is_loss_grad_op
from .common import is_optimizer_op as is_optimizer_op
from .meta_optimizer_base import MetaOptimizerBase as MetaOptimizerBase

class RawProgramOptimizer(MetaOptimizerBase):
    inner_opt: Any = ...
    meta_optimizers_white_list: Any = ...
    meta_optimizers_black_list: Any = ...
    global_ring_id: int = ...
    def __init__(self, optimizer: Any) -> None: ...
    endpoints: Any = ...
    current_endpoint: Any = ...
    rank: Any = ...
    nranks: Any = ...
    startup_program: Any = ...
    main_program: Any = ...
    def minimize_impl(
        self,
        loss: Any,
        startup_program: Optional[Any] = ...,
        parameter_list: Optional[Any] = ...,
        no_grad_set: Optional[Any] = ...,
    ): ...



from __future__ import annotations

from typing import Any

class TrainerRuntimeConfig:
    mode: Any = ...
    runtime_configs: Any = ...
    def __init__(self) -> None: ...
    def get_communicator_flags(self): ...
    def display(self, configs: Any): ...

class PSLibRuntimeConfig:
    runtime_configs: Any = ...
    def __init__(self) -> None: ...
    def get_runtime_configs(self): ...

class DistributedStrategy:
    debug_opt: Any = ...
    use_ps_gpu: bool = ...
    def __init__(self) -> None: ...
    def set_debug_opt(self, opt_info: Any) -> None: ...
    def get_debug_opt(self): ...
    def get_program_config(self): ...
    def set_program_config(self, config: Any) -> None: ...
    def check_program_config(self) -> None: ...
    def get_trainer_runtime_config(self): ...
    def set_trainer_runtime_config(self, config: Any) -> None: ...
    def check_trainer_runtime_config(self) -> None: ...
    def get_pslib_runtime_config(self): ...
    def set_pslib_runtime_config(self, config: Any) -> None: ...
    def get_server_runtime_config(self): ...
    def set_server_runtime_config(self, config: Any) -> None: ...
    def check_server_runtime_config(self) -> None: ...
    def get_execute_strategy(self): ...
    def set_execute_strategy(self, config: Any) -> None: ...
    def check_execute_strategy(self) -> None: ...
    def get_build_strategy(self): ...
    def set_build_strategy(self, config: Any) -> None: ...
    def check_build_strategy(self) -> None: ...

class SyncStrategy(DistributedStrategy):
    def __init__(self) -> None: ...
    def check_trainer_runtime_config(self) -> None: ...
    def check_program_config(self) -> None: ...
    def check_server_runtime_config(self) -> None: ...
    def check_execute_strategy(self) -> None: ...
    def check_build_strategy(self) -> None: ...

class AsyncStrategy(DistributedStrategy):
    def __init__(self) -> None: ...
    def check_trainer_runtime_config(self) -> None: ...
    def check_program_config(self) -> None: ...
    def check_server_runtime_config(self) -> None: ...
    def check_execute_strategy(self) -> None: ...
    def check_build_strategy(self) -> None: ...

class HalfAsyncStrategy(DistributedStrategy):
    def __init__(self) -> None: ...
    def check_trainer_runtime_config(self) -> None: ...
    def check_program_config(self) -> None: ...
    def check_server_runtime_config(self) -> None: ...
    def check_execute_strategy(self) -> None: ...
    def check_build_strategy(self) -> None: ...

class GeoStrategy(DistributedStrategy):
    def __init__(self, update_frequency: int = ...) -> None: ...
    def check_program_config(self) -> None: ...
    def check_trainer_runtime_config(self) -> None: ...
    def check_server_runtime_config(self) -> None: ...
    def check_execute_strategy(self) -> None: ...
    def check_build_strategy(self) -> None: ...

class StrategyFactory:
    @staticmethod
    def create_sync_strategy(): ...
    @staticmethod
    def create_half_async_strategy(): ...
    @staticmethod
    def create_async_strategy(): ...
    @staticmethod
    def create_geo_strategy(update_frequency: int = ...): ...



from __future__ import annotations

from typing import Any, Optional

from ..meta_optimizers.dygraph_optimizer import (
    HybridParallelGradScaler as HybridParallelGradScaler,
)
from ..meta_optimizers.dygraph_optimizer import (
    HybridParallelOptimizer as HybridParallelOptimizer,
)
from ..utils.hybrid_parallel_util import (
    broadcast_dp_parameters as broadcast_dp_parameters,
)
from ..utils.hybrid_parallel_util import (
    broadcast_mp_parameters as broadcast_mp_parameters,
)
from ..utils.hybrid_parallel_util import (
    broadcast_sharding_parameters as broadcast_sharding_parameters,
)
from ..utils.log_util import logger as logger
from .meta_parallel_base import MetaParallelBase as MetaParallelBase
from .parallel_layers.pp_layers import PipelineLayer as PipelineLayer
from .pp_utils.utils import is_float_tensor as is_float_tensor

class PipelineParallel(MetaParallelBase):
    use_data_parallel: Any = ...
    use_model_parallel: Any = ...
    use_sharding_parallel: Any = ...
    total_loss: Any = ...
    micro_batch_size: Any = ...
    accumulate_steps: Any = ...
    num_stages: Any = ...
    stage_id: Any = ...
    pp_group: Any = ...
    is_first_stage: Any = ...
    is_last_stage: Any = ...
    global_rank: Any = ...
    micro_batch_id: int = ...
    def __init__(self, layers: Any, hcg: Any, strategy: Any) -> None: ...
    scaler: Any = ...
    data: Any = ...
    def forward_backward_pipeline(self, data: Any, scaler: Optional[Any] = ...): ...
    optimizer: Any = ...
    lr_scheduler: Any = ...
    def train_batch(
        self, data: Any, optimizer: Any, lr_scheduler: Optional[Any] = ..., scaler: Optional[Any] = ...
    ): ...
    train_loss: Any = ...
    def eval_batch(self, data: Any, compute_loss: bool = ...): ...



from __future__ import annotations

from typing import Any

from ... import Tensor, nn
from ..ops import ConvNormActivation as ConvNormActivation

model_urls: Any

class SqueezeExcitation(nn.Layer):
    avgpool: Any = ...
    fc1: Any = ...
    fc2: Any = ...
    activation: Any = ...
    scale_activation: Any = ...
    def __init__(
        self, input_channels: Any, squeeze_channels: Any, activation: Any = ..., scale_activation: Any = ...
    ) -> None: ...
    def forward(self, input: Any) -> Tensor: ...

class InvertedResidualConfig:
    in_channels: Any = ...
    kernel: Any = ...
    expanded_channels: Any = ...
    out_channels: Any = ...
    use_se: Any = ...
    activation_layer: Any = ...
    stride: Any = ...
    def __init__(
        self,
        in_channels: Any,
        kernel: Any,
        expanded_channels: Any,
        out_channels: Any,
        use_se: Any,
        activation: Any,
        stride: Any,
        scale: float = ...,
    ) -> None: ...
    @staticmethod
    def adjust_channels(channels: Any, scale: float = ...) -> int: ...

class InvertedResidual(nn.Layer):
    use_res_connect: Any = ...
    use_se: Any = ...
    expand: Any = ...
    expand_conv: Any = ...
    bottleneck_conv: Any = ...
    mid_se: Any = ...
    linear_conv: Any = ...
    def __init__(
        self,
        in_channels: Any,
        expanded_channels: Any,
        out_channels: Any,
        filter_size: Any,
        stride: Any,
        use_se: Any,
        activation_layer: Any,
        norm_layer: Any,
    ) -> None: ...
    def forward(self, x: Any) -> Tensor: ...

class MobileNetV3(nn.Layer):
    config: Any = ...
    scale: Any = ...
    last_channel: Any = ...
    num_classes: Any = ...
    with_pool: Any = ...
    firstconv_in_channels: Any = ...
    lastconv_in_channels: Any = ...
    lastconv_out_channels: Any = ...
    conv: Any = ...
    blocks: Any = ...
    lastconv: Any = ...
    avgpool: Any = ...
    classifier: Any = ...
    def __init__(
        self, config: Any, last_channel: Any, scale: float = ..., num_classes: int = ..., with_pool: bool = ...
    ) -> None: ...
    def forward(self, x: Tensor) -> Tensor: ...
    __call__ = forward

class MobileNetV3Small(MobileNetV3):
    def __init__(
        self,
        scale: float = ...,
        num_classes: int = ...,
        with_pool: bool = ...,
    ) -> None: ...

class MobileNetV3Large(MobileNetV3):
    def __init__(
        self,
        scale: float = ...,
        num_classes: int = ...,
        with_pool: bool = ...,
    ) -> None: ...

def mobilenet_v3_small(
    pretrained: bool = ...,
    scale: float = ...,
    *,
    num_classes: int = ...,
    with_pool: bool = ...,
) -> MobileNetV3Small: ...
def mobilenet_v3_large(
    pretrained: bool = ...,
    scale: float = ...,
    *,
    num_classes: int = ...,
    with_pool: bool = ...,
) -> MobileNetV3Large: ...



from __future__ import annotations

from typing import Any, Optional

class _IterableDatasetStopIteration:
    worker_id: Any = ...
    def __init__(self, worker_id: Any) -> None: ...

class _ResumeIteration: ...

class _DatasetKind:
    MAP: int = ...
    ITER: int = ...
    @staticmethod
    def create_fetcher(kind: Any, dataset: Any, auto_collate_batch: Any, collate_fn: Any, drop_last: Any): ...

class ParentWatchDog:
    def __init__(self) -> None: ...
    def is_alive(self): ...

def get_worker_info(): ...

class WorkerInfo:
    def __init__(self, **kwargs: Any) -> None: ...
    def __setattr__(self, key: Any, val: Any): ...

class _WorkerException:
    worker_id: Any = ...
    exc_type: Any = ...
    exc_msg: Any = ...
    def __init__(self, worker_id: Any, exc_info: Optional[Any] = ...) -> None: ...
    def reraise(self) -> None: ...



from __future__ import annotations

from typing import Any

from .controller import Controller as Controller

class CollectiveController(Controller):
    @classmethod
    def enable(cls, ctx: Any): ...
    def build_pod(self): ...

class CollectiveElasticController(CollectiveController):
    @classmethod
    def enable(cls, ctx: Any): ...
    def register(self) -> None: ...
    def run(self) -> None: ...



from __future__ import annotations

from typing import Any, Optional

import paddle

from . import core as core
from . import unique_name as unique_name
from .framework import OpProtoHolder as OpProtoHolder

def get_data_vars(program: Any): ...
def apply_build_strategy(main_program: Any, startup_program: Any, build_strategy: Any, pass_attrs: Any): ...

class RegisterPassHelper:
    def __init__(self, pass_pairs: Any, pass_type: Any = ..., input_specs: Any = ...) -> None: ...
    def SerializeMultiPassDesc(self): ...

class PassDesc:
    class AttrHelper:
        def __init__(self, obj: Any, name: Any, element_index: Optional[Any] = ...) -> None: ...
        def __getitem__(self, index: Any): ...
        def __sub__(self, value: Any): ...
        def __add__(self, value: Any): ...
        def Mod(self, value: Any): ...
        def Size(self): ...
        def EQ(self, value: Any) -> None: ...
        def MappedPattern(
            self,
            var: Optional[Any] = ...,
            op: Optional[Any] = ...,
            index: int = ...,
            name: Optional[Any] = ...,
            element_index: Optional[Any] = ...,
        ): ...

    class VarHelper(paddle.static.Variable):
        def __init__(self, *args: Any, **kwargs: Any) -> None: ...
        def __getattr__(self, name: Any): ...
        def Attr(self, name: Any): ...

    class OpHelper:
        def __init__(self, type: Optional[Any] = ...) -> None: ...
        def __getattr__(self, name: Any): ...
        def __call__(self, *args: Any, **kwargs: Any): ...
        def Init(self) -> None: ...
        def Attr(self, name: Any): ...
        def SetAttr(self, name: Any, value: Any) -> None: ...
        def Output(self, name: Any): ...
        def Outputs(self): ...
        def SetOutputs(self, **kwargs: Any) -> None: ...
    OP: Any = ...

def RegisterPass(function: Optional[Any] = ..., input_specs: Any = ...): ...



from __future__ import annotations

from typing import Any, Optional

class LoggerFactory:
    @staticmethod
    def build_logger(name: Optional[Any] = ..., level: Any = ...): ...

logger: Any

def layer_to_str(base: Any, *args: Any, **kwargs: Any): ...



from __future__ import annotations

import unittest
from typing import Any, Optional

def parse_args(): ...

class TestLstmModelPTQ(unittest.TestCase):
    def get_warmup_tensor(self, data_path: Any, place: Any): ...
    def set_config(
        self,
        model_path: Any,
        num_threads: Any,
        mkldnn_cache_capacity: Any,
        warmup_data: Optional[Any] = ...,
        use_analysis: bool = ...,
        enable_ptq: bool = ...,
    ): ...
    def run_program(
        self,
        model_path: Any,
        data_path: Any,
        num_threads: Any,
        mkldnn_cache_capacity: Any,
        warmup_iter: Any,
        use_analysis: bool = ...,
        enable_ptq: bool = ...,
    ): ...
    def test_lstm_model(self) -> None: ...



from __future__ import annotations

from typing import Any, Optional

from .. import core as core
from ..framework import Parameter as Parameter
from ..layer_helper_base import LayerHelperBase as LayerHelperBase
from ..param_attr import ParamAttr as ParamAttr

class LayerObjectHelper(LayerHelperBase):
    def __init__(self, name: Any) -> None: ...
    def append_op(
        self,
        type: Optional[Any] = ...,
        inputs: Optional[Any] = ...,
        outputs: Optional[Any] = ...,
        attrs: Optional[Any] = ...,
        stop_gradient: Optional[Any] = ...,
    ): ...
    def iter_inputs_and_params(self, inputs_in: Any, param_attr_in: Optional[Any] = ...) -> None: ...
    def input_dtype(self, inputs_in: Any): ...
    def get_parameter(self, name: Any): ...
    def append_activation(self, input_var: Any, act: Optional[Any] = ..., use_cudnn: Optional[Any] = ...): ...
    def is_instance(self, param: Any, cls: Any) -> None: ...



from __future__ import annotations

from typing import Any

def data_layer_not_check(name: Any, shape: Any, dtype: str = ..., lod_level: int = ...): ...
def to_static_variable_gast_node(name: Any): ...
def create_static_variable_gast_node(name: Any): ...
def create_fill_constant_node(name: Any, value: Any): ...
def to_static_variable(x: Any): ...
def create_bool_as_type(x: Any, value: bool = ...): ...



from __future__ import annotations

from typing import Any, Optional

import numpy as np
from paddle.framework import ParamAttr
from paddle.incubate.nn import functional as incubate_f
from paddle.nn import Layer
from paddle.nn import functional as F

from ...._typing import Tensor

class FusedBiasDropoutResidualLayerNorm(Layer):
    def __init__(
        self,
        embed_dim: int,
        dropout_rate: float = ...,
        weight_attr: Optional[ParamAttr] = ...,
        bias_attr: Optional[ParamAttr] = ...,
        epsilon: float = 1e-5,
        name: Optional[str] = ...,
    ) -> None: ...
    def forward(self, x, residual) -> Tensor: ...
    def extra_repr(self) -> str: ...

class FusedMultiHeadAttention(Layer):
    def __init__(
        self,
        embed_dim,
        num_heads,
        dropout_rate=0.5,
        attn_dropout_rate=0.5,
        kdim=None,
        vdim=None,
        normalize_before=False,
        need_weights=False,
        qkv_weight_attr=None,
        qkv_bias_attr=None,
        linear_weight_attr=None,
        linear_bias_attr=None,
        pre_ln_scale_attr=None,
        pre_ln_bias_attr=None,
        ln_scale_attr=None,
        ln_bias_attr=None,
        epsilon=1e-5,
        nranks=1,
        ring_id=-1,
        name=None,
    ): ...
    def forward(self, query, key=None, value=None, attn_mask=None, cache=None): ...
    def extra_repr(self) -> str: ...

class FusedFeedForward(Layer):
    def __init__(
        self,
        d_model,
        dim_feedforward,
        dropout_rate=0.1,
        epsilon=1e-05,
        activation="relu",
        act_dropout_rate=None,
        normalize_before=False,
        linear1_weight_attr=None,
        linear1_bias_attr=None,
        linear2_weight_attr=None,
        linear2_bias_attr=None,
        ln1_scale_attr=None,
        ln1_bias_attr=None,
        ln2_scale_attr=None,
        ln2_bias_attr=None,
        nranks=1,
        ring_id=-1,
        name=None,
    ): ...
    def forward(self, src, cache=None) -> Tensor: ...
    def extra_repr(self) -> str:
        name_str = ", name={}".format(self.name) if self.name else ""
        return "d_model={}, dim_feedforward={}, dropout_rate={}, epsilon={}, activation={}, act_dropout_rate={}, normalize_before={}, dtype={}{}".format(
            self._d_model,
            self._dim_feedforward,
            self._dropout_rate,
            self._epsilon,
            self._act_method,
            self._act_dropout_rate,
            self._normalize_before,
            self._dtype,
            name_str,
        )
    def _amp_decorate(self, dtype): ...

class FusedTransformerEncoderLayer(Layer):
    def __init__(
        self,
        d_model,
        nhead,
        dim_feedforward,
        dropout_rate=0.1,
        activation="relu",
        attn_dropout_rate=None,
        act_dropout_rate=None,
        normalize_before=False,
        weight_attr=None,
        bias_attr=None,
    ): ...
    def forward(self, src, src_mask=None, cache=None): ...

class FusedTransformer(Layer):
    def __init__(
        self,
        d_model: int = ...,
        nhead: int = ...,
        num_encoder_layers: int = ...,
        num_decoder_layers: int = ...,
        dim_feedforward: int = ...,
        dropout: float = ...,
        activation: str = ...,
        attn_dropout: Optional[Any] = ...,
        act_dropout: Optional[Any] = ...,
        normalize_before: bool = ...,
        weight_attr: Optional[Any] = ...,
        bias_attr: Optional[Any] = ...,
        custom_encoder: Optional[Any] = ...,
        custom_decoder: Optional[Any] = ...,
    ): ...
    def forward(
        self, src, tgt, src_mask: Optional[Any] = ..., tgt_mask: Optional[Any] = ..., memory_mask: Optional[Any] = ...
    ): ...

class FusedMultiTransformer(Layer):
    def __init__(
        self,
        embed_dim,
        num_heads,
        dim_feedforward,
        dropout_rate=0.0,
        activation="gelu",
        normalize_before=True,
        ln_scale_attrs: Optional[ParamAttr] = None,
        ln_bias_attrs: Optional[ParamAttr] = None,
        qkv_weight_attrs: Optional[ParamAttr] = None,
        qkv_bias_attrs: Optional[ParamAttr] = None,
        linear_weight_attrs: Optional[ParamAttr] = None,
        linear_bias_attrs: Optional[ParamAttr] = None,
        ffn_ln_scale_attrs: Optional[ParamAttr] = None,
        ffn_ln_bias_attrs: Optional[ParamAttr] = None,
        ffn1_weight_attrs: Optional[ParamAttr] = None,
        ffn1_bias_attrs: Optional[ParamAttr] = None,
        ffn2_weight_attrs: Optional[ParamAttr] = None,
        ffn2_bias_attrs: Optional[ParamAttr] = None,
        epsilon: float = ...,
        num_layers: int = ...,
        nranks: int = ...,
        trans_qkvw: bool = ...,
        ring_id: int = ...,
        name: Optional[str] = ...,
    ) -> None: ...
    def forward(self, src, attn_mask=None, caches=None, time_step=None): ...



from __future__ import annotations

from .tensor import inverse as inv
from .tensor.linalg import cholesky as cholesky
from .tensor.linalg import cholesky_solve as cholesky_solve
from .tensor.linalg import cond as cond
from .tensor.linalg import cov as cov
from .tensor.linalg import det as det
from .tensor.linalg import eig as eig
from .tensor.linalg import eigh as eigh
from .tensor.linalg import eigvals as eigvals
from .tensor.linalg import eigvalsh as eigvalsh
from .tensor.linalg import lstsq as lstsq
from .tensor.linalg import lu as lu
from .tensor.linalg import lu_unpack as lu_unpack
from .tensor.linalg import matrix_power as matrix_power
from .tensor.linalg import matrix_rank as matrix_rank
from .tensor.linalg import multi_dot as multi_dot
from .tensor.linalg import norm as norm
from .tensor.linalg import pinv as pinv
from .tensor.linalg import qr as qr
from .tensor.linalg import slogdet as slogdet
from .tensor.linalg import solve as solve
from .tensor.linalg import svd as svd
from .tensor.linalg import triangular_solve as triangular_solve

__all__ = [
    "inv",  # re-export
]



from __future__ import annotations

from . import ast_transformer as ast_transformer
from . import convert_call_func as convert_call_func
from . import convert_operators as convert_operators
from . import logging_utils as logging_utils
from . import loop_transformer as loop_transformer
from . import program_translator as program_translator
from . import static_analysis as static_analysis
from . import variable_trans_func as variable_trans_func
from .ast_transformer import *
from .convert_call_func import *
from .logging_utils import *
from .loop_transformer import *
from .program_translator import *
from .static_analysis import *
from .variable_trans_func import *



from __future__ import annotations

import unittest

from paddle.fluid.dygraph.nn import Pool2D as Pool2D
from paddle.nn import BatchNorm as BatchNorm
from paddle.nn import Conv2D as Conv2D
from paddle.nn import Linear as Linear
from paddle.nn import Softmax as Softmax
from paddle.nn.layer import LeakyReLU as LeakyReLU
from paddle.nn.layer import ReLU as ReLU
from paddle.nn.layer import ReLU6 as ReLU6
from paddle.nn.layer import Sigmoid as Sigmoid

class TestImperativeOutSclae(unittest.TestCase):
    def func_out_scale_acc(self) -> None: ...
    def test_out_scale_acc(self) -> None: ...



from __future__ import annotations

from typing import Any, Optional

class Variable:
    def __init__(self, is_discrete: bool = ..., event_rank: int = ..., constraint: Optional[Any] = ...) -> None: ...
    @property
    def is_discrete(self): ...
    @property
    def event_rank(self): ...
    def constraint(self, value: Any): ...

class Real(Variable):
    def __init__(self, event_rank: int = ...) -> None: ...

class Positive(Variable):
    def __init__(self, event_rank: int = ...) -> None: ...

class Independent(Variable):
    def __init__(self, base: Any, reinterpreted_batch_rank: Any) -> None: ...
    def constraint(self, value: Any): ...

class Stack(Variable):
    def __init__(self, vars: Any, axis: int = ...) -> None: ...
    @property
    def is_discrete(self): ...
    @property
    def event_rank(self): ...
    def constraint(self, value: Any): ...

real: Any
positive: Any



from __future__ import annotations

from typing import Any, Optional

from .. import Tensor
from .._typing import DTypeLike, ShapeLike
from ..fluid.data_feeder import check_dtype as check_dtype
from ..fluid.data_feeder import check_shape as check_shape
from ..fluid.data_feeder import check_type as check_type
from ..fluid.data_feeder import check_variable_and_dtype as check_variable_and_dtype
from ..fluid.layer_helper import LayerHelper as LayerHelper
from ..fluid.layers import utils as utils
from ..framework import convert_np_dtype_to_dtype_ as convert_np_dtype_to_dtype_
from ..framework import core as core
from ..framework import dygraph_only as dygraph_only

def bernoulli(x: Any, name: Optional[Any] = ...): ...
def poisson(x: Any, name: Optional[Any] = ...): ...
def multinomial(x: Any, num_samples: int = ..., replacement: bool = ..., name: Optional[Any] = ...): ...
def gaussian(
    shape: Any, mean: float = ..., std: float = ..., dtype: Optional[Any] = ..., name: Optional[Any] = ...
): ...
def standard_normal(shape: Any, dtype: Optional[Any] = ..., name: Optional[Any] = ...): ...
def randn(
    shape: Any,
    dtype: Optional[DTypeLike] = ...,
    name: Optional[Any] = ...,
) -> Tensor: ...
def normal(mean: float = ..., std: float = ..., shape: Optional[Any] = ..., name: Optional[Any] = ...): ...
def uniform(
    shape: Any, dtype: Optional[Any] = ..., min: Any = ..., max: float = ..., seed: int = ..., name: Optional[Any] = ...
): ...
def uniform_(x: Any, min: Any = ..., max: float = ..., seed: int = ..., name: Optional[Any] = ...): ...
def randint(
    low: int = ..., high: Optional[Any] = ..., shape: Any = ..., dtype: Optional[Any] = ..., name: Optional[Any] = ...
): ...
def randint_like(
    x: Any, low: int = ..., high: Optional[Any] = ..., dtype: Optional[Any] = ..., name: Optional[Any] = ...
): ...
def randperm(n: Any, dtype: str = ..., name: Optional[Any] = ...): ...
def rand(
    shape: ShapeLike,
    dtype: Optional[DTypeLike] = ...,
    name: Optional[str] = ...,
) -> Tensor: ...
def exponential_(x: Any, lam: float = ..., name: Optional[Any] = ...): ...



from __future__ import annotations

from typing import Any, Optional

from .meta_optimizer_base import MetaOptimizerBase as MetaOptimizerBase

class DGCOptimizer(MetaOptimizerBase):
    inner_opt: Any = ...
    dgc_opt: Any = ...
    meta_optimizers_white_list: Any = ...
    meta_optimizers_black_list: Any = ...
    def __init__(self, optimizer: Any) -> None: ...
    def backward(
        self,
        loss: Any,
        startup_program: Optional[Any] = ...,
        parameter_list: Optional[Any] = ...,
        no_grad_set: Optional[Any] = ...,
        callbacks: Optional[Any] = ...,
    ): ...
    def apply_gradients(self, params_grads: Any): ...
    def apply_optimize(self, loss: Any, startup_program: Any, params_grads: Any): ...
    def minimize_impl(
        self,
        loss: Any,
        startup_program: Optional[Any] = ...,
        parameter_list: Optional[Any] = ...,
        no_grad_set: Optional[Any] = ...,
    ): ...



from __future__ import annotations

from typing import Any

from .ptq_quantizer import *

class PTQConfig:
    in_act_quantizer: Any = ...
    out_act_quantizer: Any = ...
    wt_quantizer: Any = ...
    quant_hook_handle: Any = ...
    enable_in_act_quantizer: bool = ...
    def __init__(self, activation_quantizer: Any, weight_quantizer: Any) -> None: ...

default_ptq_config: Any



from __future__ import annotations

from typing import Any, Optional

class ProgramDeps:
    def __init__(self, block: Any, start_vars: Any, end_vars: Any) -> None: ...
    def get_sub_block_deps(self, idx: Any): ...
    def get_var_deps(self, var_name: Any): ...
    def crop_input_var_from_op(self, op_idx: Any, var_name: Any) -> None: ...
    def crop_output_var_from_op(self, op_idx: Any, var_name: Any) -> None: ...
    def remove_op(self, op_idx: Any, reserved_vars: Optional[Any] = ...) -> None: ...
    def should_remove_op(self, op_idx: Any): ...



from __future__ import annotations

from typing import Any, Optional

from ...fluid import framework as framework
from ...fluid.data_feeder import check_variable_and_dtype as check_variable_and_dtype
from ...fluid.initializer import Initializer as Initializer
from ...tensor import diag as diag
from ...tensor import qr as qr
from ...tensor import reshape as reshape
from ...tensor import sign as sign
from ...tensor import transpose as transpose

class Orthogonal(Initializer):
    def __init__(self, gain: float = ..., name: Optional[Any] = ...) -> None: ...
    def __call__(self, var: Any, block: Optional[Any] = ...): ...



from __future__ import annotations

from typing import Any

from ..ps.utils.public import *
from .pass_base import PassBase as PassBase
from .pass_base import register_pass as register_pass

class AppendSendOpsPass(PassBase):
    def __init__(self) -> None: ...

class DistributedOpsPass(PassBase):
    w_2_table_id: Any = ...
    emb_size: Any = ...
    def __init__(self) -> None: ...

class DeleteOptimizesPass(PassBase):
    def __init__(self) -> None: ...

class DeleteExtraOptimizerPass(PassBase):
    def __init__(self) -> None: ...

class FakeInitOpsPass(PassBase):
    def __init__(self) -> None: ...

class PsGpuPass(PassBase):
    def __init__(self) -> None: ...

class PsTranspilePass(PassBase):
    def __init__(self) -> None: ...

class SplitHeterWorkerOpsPass(PassBase):
    def __init__(self) -> None: ...

class SplitTrainerOpsPass(PassBase):
    def __init__(self) -> None: ...

class SetHeterPipelineOptPass(PassBase):
    def __init__(self) -> None: ...



from __future__ import annotations

from typing import Any, Optional

class HeterParallelOptimizer:
    def __init__(self, optimizer: Any, strategy: Any) -> None: ...
    def step(self) -> None: ...
    def minimize(
        self,
        loss: Any,
        startup_program: Optional[Any] = ...,
        parameters: Optional[Any] = ...,
        no_grad_set: Optional[Any] = ...,
    ): ...
    def __getattr__(self, item: Any): ...



from __future__ import annotations

from typing import Any, Optional

from ...fluid.layers.tensor import fill_constant as fill_constant

def get_all_process_groups(): ...
def get_process_group(group_id: Any, g_process_group_map: Optional[Any] = ...): ...
def get_world_process_group(): ...
def new_process_group(ranks: Any): ...

class ProcessGroup:
    def __init__(self, group_id: Any, ranks: Any) -> None: ...
    @property
    def id(self): ...
    @property
    def ranks(self): ...
    @property
    def nranks(self): ...
    def add_ranks(self, new_ranks: Any) -> None: ...
    def local_rank(self, global_rank: Any): ...
    def is_instantiate(self): ...
    def instantiate(self) -> None: ...



from __future__ import annotations

from .base.distributed_strategy import DistributedStrategy as DistributedStrategy
from .base.fleet_base import Fleet as Fleet
from .base.role_maker import PaddleCloudRoleMaker as PaddleCloudRoleMaker
from .base.role_maker import Role as Role
from .base.role_maker import UserDefinedRoleMaker as UserDefinedRoleMaker
from .base.topology import CommunicateTopology as CommunicateTopology
from .base.topology import HybridCommunicateGroup as HybridCommunicateGroup
from .base.util_factory import UtilBase as UtilBase
from .data_generator.data_generator import (
    MultiSlotDataGenerator as MultiSlotDataGenerator,
)
from .data_generator.data_generator import (
    MultiSlotStringDataGenerator as MultiSlotStringDataGenerator,
)

rank_in_node = local_rank



from __future__ import annotations

from typing import Any, Optional

from ..fluid.data_feeder import check_dtype as check_dtype
from ..fluid.data_feeder import check_type as check_type
from ..fluid.data_feeder import check_variable_and_dtype as check_variable_and_dtype
from ..fluid.data_feeder import convert_dtype as convert_dtype
from ..fluid.layer_helper import LayerHelper as LayerHelper
from ..framework import core as core
from ..static import Variable as Variable
from .search import where as where

def mean(x: Any, axis: Optional[Any] = ..., keepdim: bool = ..., name: Optional[Any] = ...): ...
def var(x: Any, axis: Optional[Any] = ..., unbiased: bool = ..., keepdim: bool = ..., name: Optional[Any] = ...): ...
def std(x: Any, axis: Optional[Any] = ..., unbiased: bool = ..., keepdim: bool = ..., name: Optional[Any] = ...): ...
def numel(x: Any, name: Optional[Any] = ...): ...
def median(x: Any, axis: Optional[Any] = ..., keepdim: bool = ..., name: Optional[Any] = ...): ...
def quantile(x: Any, q: Any, axis: Optional[Any] = ..., keepdim: bool = ...): ...



from __future__ import annotations

from typing import Any, Optional

from ..common import OP_ROLE_KEY as OP_ROLE_KEY
from ..common import OpRole as OpRole
from ..common import is_optimizer_op as is_optimizer_op
from ..common import is_update_op as is_update_op
from .shard import Shard as Shard

class PlaceType:
    CPU: int = ...
    CUDA: int = ...
    CUDA_PINNED: int = ...
    XPU: int = ...
    NPU: int = ...
    NPU_PINNED: int = ...
    @staticmethod
    def default_device(): ...
    @staticmethod
    def default_pinned(): ...

class OffloadHelper:
    cpu_place_type: int = ...
    cuda_place_type: Any = ...
    cuda_pinned_place_type: Any = ...
    mp_ring_id: Any = ...
    dp_ring_id: Any = ...
    def __init__(self, mp_ring_id: Optional[Any] = ..., dp_ring_id: Optional[Any] = ...) -> None: ...
    def offload_fp32param(self, block: Any, startup_block: Any, offload: bool = ...) -> None: ...
    def cast_fp32param_in_optimize(self, block: Any, startup_block: Any) -> None: ...
    def offload(self, block: Any, startup_block: Any) -> None: ...
    def opt_sharding_cast_fp32param(self, block: Any, startup_block: Any, params: Any, offload: bool = ...) -> None: ...



from __future__ import annotations

from typing import Any

logger: Any
formatter: Any
ch: Any
ELASTIC_EXIT_CODE: int
ELASTIC_AUTO_PARALLEL_EXIT_CODE: int
ELASTIC_TIMEOUT: Any
ELASTIC_TTL: int

class ElasticLevel:
    FAULT_TOLERANCE: int = ...
    ELASTIC: int = ...

class ElasticStatus:
    COMPLETED: str = ...
    ERROR: str = ...
    HOLD: str = ...
    RESTART: str = ...
    EXIT: str = ...

class LauncherInterface:
    args: Any = ...
    procs: Any = ...
    def __init__(self, args: Any) -> None: ...
    def launch(self) -> None: ...
    def stop(self) -> None: ...
    def watch(self) -> None: ...

class ElasticManager:
    args: Any = ...
    host: Any = ...
    elastic_timeout: Any = ...
    start_port: Any = ...
    trainers: Any = ...
    np: Any = ...
    dist_endpoints: Any = ...
    trainer_endpoints_list: Any = ...
    curr_host: Any = ...
    elastic_level: Any = ...
    hosts: Any = ...
    stopped: bool = ...
    sigint: int = ...
    need_sync: bool = ...
    elastic_startup_time: Any = ...
    enable: bool = ...
    etcd: Any = ...
    prefix: Any = ...
    node_prefix: Any = ...
    np_path: Any = ...
    endpoints_path: Any = ...
    host_path: Any = ...
    watches: Any = ...
    launcher: Any = ...
    def __init__(self, args: Any, etcd_client: Any) -> None: ...
    def exit(self, completed: bool = ...) -> None: ...
    def pre_hook(self) -> None: ...
    def wait(self) -> None: ...
    def run(self, launcher: Any) -> None: ...
    def watch(self): ...
    def signal_handler(self, sigint: Any, frame: Any) -> None: ...



from __future__ import annotations

from typing import Any, Optional

from paddle.nn import Layer

class FakeQuantAbsMax(Layer):
    def __init__(
        self, name: Optional[Any] = ..., quant_bits: int = ..., dtype: str = ..., quant_on_weight: bool = ...
    ) -> None: ...
    def forward(self, input: Any): ...

class FakeQuantMovingAverageAbsMax(Layer):
    def __init__(
        self, name: Optional[Any] = ..., moving_rate: float = ..., quant_bits: int = ..., dtype: str = ...
    ) -> None: ...
    def forward(self, input: Any): ...

class FakeQuantChannelWiseAbsMax(Layer):
    def __init__(
        self,
        name: Optional[Any] = ...,
        channel_num: Optional[Any] = ...,
        quant_bits: int = ...,
        quant_axis: int = ...,
        dtype: str = ...,
        quant_on_weight: bool = ...,
    ) -> None: ...
    def forward(self, input: Any): ...

class MovingAverageAbsMaxScale(Layer):
    def __init__(self, name: Optional[Any] = ..., moving_rate: float = ..., dtype: str = ...) -> None: ...
    def forward(self, input: Any): ...

QuantStub = MovingAverageAbsMaxScale

class QuantizedConv2D(Layer):
    weight: Any = ...
    bias: Any = ...
    def __init__(
        self,
        layer: Any,
        weight_bits: int = ...,
        activation_bits: int = ...,
        moving_rate: float = ...,
        weight_quantize_type: str = ...,
        activation_quantize_type: str = ...,
        weight_pre_layer: Optional[Any] = ...,
        act_pre_layer: Optional[Any] = ...,
        weight_quant_layer: Optional[Any] = ...,
        act_quant_layer: Optional[Any] = ...,
    ) -> None: ...
    def forward(self, input: Any): ...

class QuantizedConv2DTranspose(Layer):
    weight: Any = ...
    bias: Any = ...
    def __init__(
        self,
        layer: Any,
        weight_bits: int = ...,
        activation_bits: int = ...,
        moving_rate: float = ...,
        weight_quantize_type: str = ...,
        activation_quantize_type: str = ...,
        weight_pre_layer: Optional[Any] = ...,
        act_pre_layer: Optional[Any] = ...,
        weight_quant_layer: Optional[Any] = ...,
        act_quant_layer: Optional[Any] = ...,
    ) -> None: ...
    def forward(self, input: Any, output_size: Optional[Any] = ...): ...

class QuantizedLinear(Layer):
    weight: Any = ...
    bias: Any = ...
    name: Any = ...
    def __init__(
        self,
        layer: Any,
        weight_bits: int = ...,
        activation_bits: int = ...,
        moving_rate: float = ...,
        weight_quantize_type: str = ...,
        activation_quantize_type: str = ...,
        weight_pre_layer: Optional[Any] = ...,
        act_pre_layer: Optional[Any] = ...,
        weight_quant_layer: Optional[Any] = ...,
        act_quant_layer: Optional[Any] = ...,
    ) -> None: ...
    def forward(self, input: Any): ...

class MAOutputScaleLayer(Layer):
    def __init__(
        self, layer: Optional[Any] = ..., moving_rate: float = ..., name: Optional[Any] = ..., dtype: str = ...
    ) -> None: ...
    def forward(self, *inputs: Any, **kwargs: Any): ...

class FakeQuantMAOutputScaleLayer(Layer):
    def __init__(
        self,
        layer: Any,
        weight_bits: int = ...,
        activation_bits: int = ...,
        moving_rate: float = ...,
        name: Optional[Any] = ...,
        *args: Any,
        **kwargs: Any,
    ) -> None: ...
    def forward(self, *inputs: Any, **kwargs: Any): ...



from __future__ import annotations

from typing import Any, Optional

from paddle.distributed.collective import wait as wait
from paddle.optimizer import Optimizer

from .group_sharded_storage import GradStorage as GradStorage
from .group_sharded_storage import ParamStorage as ParamStorage
from .group_sharded_utils import GroupShardedClipGrad as GroupShardedClipGrad
from .group_sharded_utils import Type as Type
from .group_sharded_utils import device_guard as device_guard

alignment: Any
align: Any

class GroupShardedOptimizerStage2(Optimizer):
    world_size: Any = ...
    param_storages: Any = ...
    offload: Any = ...
    offload_device: str = ...
    offload_buffer_size: int = ...
    offload_param2align: Any = ...
    offload_params: Any = ...
    offload_grads: Any = ...
    dev_id: Any = ...
    def __init__(
        self,
        params: Any,
        optim: Any,
        group: Optional[Any] = ...,
        offload: bool = ...,
        device: str = ...,
        pertrain_sync_models: bool = ...,
        **kw: Any,
    ): ...
    @property
    def local_params(self): ...
    @property
    def param2rank(self): ...
    @property
    def dtype_rank_params(self): ...
    @property
    def rank_buffer_size(self): ...
    def step(self) -> None: ...
    def minimize(self) -> None: ...
    def set_state_dict(self, state_dict: Any) -> None: ...
    def state_dict(self): ...



from __future__ import annotations

from typing import Any, Optional

from paddle.static import Variable as Variable

from ..fluid.data_feeder import check_variable_and_dtype as check_variable_and_dtype
from ..fluid.framework import in_dygraph_mode as in_dygraph_mode
from ..fluid.layer_helper import LayerHelper as LayerHelper
from ..fluid.layers import rank as rank
from ..fluid.layers import shape as shape
from ..framework import core as core

def is_complex(x: Any): ...
def is_floating_point(x: Any): ...
def is_integer(x: Any): ...
def real(x: Any, name: Optional[Any] = ...): ...
def imag(x: Any, name: Optional[Any] = ...): ...



from __future__ import annotations

from typing import Any, Optional

def cuda_profiler(output_file: Any, output_mode: Optional[Any] = ..., config: Optional[Any] = ...) -> None: ...
def reset_profiler() -> None: ...
def start_profiler(state: Any, tracer_option: str = ...) -> None: ...
def stop_profiler(sorted_key: Optional[Any] = ..., profile_path: str = ...) -> None: ...
def profiler(
    state: Any, sorted_key: Optional[Any] = ..., profile_path: str = ..., tracer_option: str = ...
) -> None: ...



from __future__ import annotations

from collections import OrderedDict as OrderedDict
from typing import Any, Optional

from .pass_base import PassBase as PassBase
from .pass_base import PassType as PassType
from .pass_base import register_pass as register_pass

def find_adjacent_match_sequences(iterable: Any, filter_func: Any, adjacent_filter_func: Optional[Any] = ...): ...
def insert_fuse_all_reduce_ops(
    block: Any, reversed_op_indices: Any, input_var_names: Any, output_var_names: Any, dtype: Any, attrs: Any
): ...
def has_same_attrs(op1: Any, op2: Any, attr_names: Any): ...
def filter_all_collective_op_indices(block: Any): ...
def find_all_fuse_all_reduce_groups(block: Any): ...
def split_fuse_all_reduce_groups_by_deps(block: Any, groups: Any, op_deps: Any): ...
def insert_coalesce_tensor_ops(block: Any, coalesce_ops_kwargs: Any): ...
def insert_fuse_all_reduce_by_memory_size(block: Any, groups: Any, max_memory_size: Any) -> None: ...

class FuseAllReducePass(PassBase):
    def __init__(self) -> None: ...



from __future__ import annotations

from ..fluid.dygraph.container import LayerList as LayerList
from ..fluid.dygraph.container import ParameterList as ParameterList
from ..fluid.dygraph.container import Sequential as Sequential
from ..fluid.dygraph.layers import Layer as Layer
from .clip import ClipGradByGlobalNorm as ClipGradByGlobalNorm
from .clip import ClipGradByNorm as ClipGradByNorm
from .clip import ClipGradByValue as ClipGradByValue
from .decode import BeamSearchDecoder as BeamSearchDecoder
from .decode import dynamic_decode as dynamic_decode
from .initializer import Initializer as Initializer
from .layer.activation import CELU as CELU
from .layer.activation import ELU as ELU
from .layer.activation import GELU as GELU
from .layer.activation import SELU as SELU
from .layer.activation import Hardshrink as Hardshrink
from .layer.activation import Hardsigmoid as Hardsigmoid
from .layer.activation import Hardswish as Hardswish
from .layer.activation import Hardtanh as Hardtanh
from .layer.activation import LeakyReLU as LeakyReLU
from .layer.activation import LogSigmoid as LogSigmoid
from .layer.activation import LogSoftmax as LogSoftmax
from .layer.activation import Maxout as Maxout
from .layer.activation import Mish as Mish
from .layer.activation import PReLU as PReLU
from .layer.activation import ReLU as ReLU
from .layer.activation import ReLU6 as ReLU6
from .layer.activation import Sigmoid as Sigmoid
from .layer.activation import Silu as Silu
from .layer.activation import Softmax as Softmax
from .layer.activation import Softplus as Softplus
from .layer.activation import Softshrink as Softshrink
from .layer.activation import Softsign as Softsign
from .layer.activation import Swish as Swish
from .layer.activation import Tanh as Tanh
from .layer.activation import Tanhshrink as Tanhshrink
from .layer.activation import ThresholdedReLU as ThresholdedReLU
from .layer.common import AlphaDropout as AlphaDropout
from .layer.common import Bilinear as Bilinear
from .layer.common import CosineSimilarity as CosineSimilarity
from .layer.common import Dropout as Dropout
from .layer.common import Dropout2D as Dropout2D
from .layer.common import Dropout3D as Dropout3D
from .layer.common import Embedding as Embedding
from .layer.common import Flatten as Flatten
from .layer.common import Fold as Fold
from .layer.common import Identity as Identity
from .layer.common import Linear as Linear
from .layer.common import Pad1D as Pad1D
from .layer.common import Pad2D as Pad2D
from .layer.common import Pad3D as Pad3D
from .layer.common import Unfold as Unfold
from .layer.common import Upsample as Upsample
from .layer.common import UpsamplingBilinear2D as UpsamplingBilinear2D
from .layer.common import UpsamplingNearest2D as UpsamplingNearest2D
from .layer.common import ZeroPad2D as ZeroPad2D
from .layer.container import LayerDict as LayerDict
from .layer.conv import Conv1D as Conv1D
from .layer.conv import Conv1DTranspose as Conv1DTranspose
from .layer.conv import Conv2D as Conv2D
from .layer.conv import Conv2DTranspose as Conv2DTranspose
from .layer.conv import Conv3D as Conv3D
from .layer.conv import Conv3DTranspose as Conv3DTranspose
from .layer.distance import PairwiseDistance as PairwiseDistance
from .layer.loss import BCELoss as BCELoss
from .layer.loss import BCEWithLogitsLoss as BCEWithLogitsLoss
from .layer.loss import CrossEntropyLoss as CrossEntropyLoss
from .layer.loss import CTCLoss as CTCLoss
from .layer.loss import HingeEmbeddingLoss as HingeEmbeddingLoss
from .layer.loss import HSigmoidLoss as HSigmoidLoss
from .layer.loss import KLDivLoss as KLDivLoss
from .layer.loss import L1Loss as L1Loss
from .layer.loss import MarginRankingLoss as MarginRankingLoss
from .layer.loss import MSELoss as MSELoss
from .layer.loss import NLLLoss as NLLLoss
from .layer.loss import SmoothL1Loss as SmoothL1Loss
from .layer.norm import BatchNorm as BatchNorm
from .layer.norm import BatchNorm1D as BatchNorm1D
from .layer.norm import BatchNorm2D as BatchNorm2D
from .layer.norm import BatchNorm3D as BatchNorm3D
from .layer.norm import GroupNorm as GroupNorm
from .layer.norm import InstanceNorm1D as InstanceNorm1D
from .layer.norm import InstanceNorm2D as InstanceNorm2D
from .layer.norm import InstanceNorm3D as InstanceNorm3D
from .layer.norm import LayerNorm as LayerNorm
from .layer.norm import LocalResponseNorm as LocalResponseNorm
from .layer.norm import SpectralNorm as SpectralNorm
from .layer.norm import SyncBatchNorm as SyncBatchNorm
from .layer.pooling import AdaptiveAvgPool1D as AdaptiveAvgPool1D
from .layer.pooling import AdaptiveAvgPool2D as AdaptiveAvgPool2D
from .layer.pooling import AdaptiveAvgPool3D as AdaptiveAvgPool3D
from .layer.pooling import AdaptiveMaxPool1D as AdaptiveMaxPool1D
from .layer.pooling import AdaptiveMaxPool2D as AdaptiveMaxPool2D
from .layer.pooling import AdaptiveMaxPool3D as AdaptiveMaxPool3D
from .layer.pooling import AvgPool1D as AvgPool1D
from .layer.pooling import AvgPool2D as AvgPool2D
from .layer.pooling import AvgPool3D as AvgPool3D
from .layer.pooling import MaxPool1D as MaxPool1D
from .layer.pooling import MaxPool2D as MaxPool2D
from .layer.pooling import MaxPool3D as MaxPool3D
from .layer.pooling import MaxUnPool1D as MaxUnPool1D
from .layer.pooling import MaxUnPool2D as MaxUnPool2D
from .layer.pooling import MaxUnPool3D as MaxUnPool3D
from .layer.rnn import GRU as GRU
from .layer.rnn import LSTM as LSTM
from .layer.rnn import RNN as RNN
from .layer.rnn import BiRNN as BiRNN
from .layer.rnn import GRUCell as GRUCell
from .layer.rnn import LSTMCell as LSTMCell
from .layer.rnn import RNNCellBase as RNNCellBase
from .layer.rnn import SimpleRNN as SimpleRNN
from .layer.rnn import SimpleRNNCell as SimpleRNNCell
from .layer.transformer import MultiHeadAttention as MultiHeadAttention
from .layer.transformer import Transformer as Transformer
from .layer.transformer import TransformerDecoder as TransformerDecoder
from .layer.transformer import TransformerDecoderLayer as TransformerDecoderLayer
from .layer.transformer import TransformerEncoder as TransformerEncoder
from .layer.transformer import TransformerEncoderLayer as TransformerEncoderLayer
from .layer.vision import PixelShuffle as PixelShuffle



from __future__ import annotations

from typing import Any, Optional

from paddle.fluid import core as core

def graph_khop_sampler(
    row: Any,
    colptr: Any,
    input_nodes: Any,
    sample_sizes: Any,
    sorted_eids: Optional[Any] = ...,
    return_eids: bool = ...,
    name: Optional[Any] = ...,
): ...



from __future__ import annotations

from .base_cost import OP_COST_FACTORY as OP_COST_FACTORY
from .base_cost import Cost as Cost
from .comm_op_cost import AllreduceSumCost as AllreduceSumCost
from .comp_op_cost import MatmulV2OpCost as MatmulV2OpCost
from .estimate_cost import CostEstimator as CostEstimator
from .tensor_cost import TensorCost as TensorCost



from __future__ import annotations

from typing import Any, Optional

class PostTrainingQuantization:
    def __init__(
        self,
        executor: Optional[Any] = ...,
        scope: Optional[Any] = ...,
        model_dir: Optional[Any] = ...,
        model_filename: Optional[Any] = ...,
        params_filename: Optional[Any] = ...,
        batch_generator: Optional[Any] = ...,
        sample_generator: Optional[Any] = ...,
        data_loader: Optional[Any] = ...,
        batch_size: int = ...,
        batch_nums: Optional[Any] = ...,
        algo: str = ...,
        hist_percent: float = ...,
        quantizable_op_type: Any = ...,
        round_type: str = ...,
        learning_rate: float = ...,
        is_full_quantize: bool = ...,
        bias_correction: bool = ...,
        activation_bits: int = ...,
        weight_bits: int = ...,
        activation_quantize_type: str = ...,
        weight_quantize_type: str = ...,
        onnx_format: bool = ...,
        optimize_model: bool = ...,
        is_use_cache_file: bool = ...,
        skip_tensor_list: Optional[Any] = ...,
        cache_dir: Optional[Any] = ...,
    ) -> None: ...
    def quantize(self): ...
    def save_quantized_model(
        self, save_model_path: Any, model_filename: Optional[Any] = ..., params_filename: Optional[Any] = ...
    ) -> None: ...

class WeightQuantization:
    def __init__(
        self, model_dir: Any, model_filename: Optional[Any] = ..., params_filename: Optional[Any] = ...
    ) -> None: ...
    def quantize_weight_to_int(
        self,
        save_model_dir: Any,
        save_model_filename: Optional[Any] = ...,
        save_params_filename: Optional[Any] = ...,
        quantizable_op_type: Any = ...,
        weight_bits: int = ...,
        weight_quantize_type: str = ...,
        generate_test_model: bool = ...,
        threshold_rate: float = ...,
    ) -> None: ...
    def convert_weight_to_fp16(self, save_model_dir: Any) -> None: ...



from __future__ import annotations

from typing import Any

class JobMode:
    COLLECTIVE: str = ...
    PS: str = ...
    HETER: str = ...

class Job:
    def __init__(self, jid: str = ..., mode: Any = ..., nnodes: str = ...) -> None: ...
    @property
    def mode(self): ...
    @property
    def id(self): ...
    @property
    def elastic(self): ...
    @property
    def replicas(self): ...
    @property
    def replicas_min(self): ...
    @property
    def replicas_max(self): ...
    @replicas.setter
    def replicas(self, replicas: Any) -> None: ...
    def set_replicas(self, nnodes: str) -> Any: ...



from __future__ import annotations

from typing import Any

def create_lod_tensor(data: Any, recursive_seq_lens: Any, place: Any): ...
def create_random_int_lodtensor(recursive_seq_lens: Any, base_shape: Any, place: Any, low: Any, high: Any): ...



from __future__ import annotations

from typing import Any, Optional

from .._typing import DTypeLike, ShapeLike, Tensor, _DTypeString
from .framework import Variable

def convert_dtype(dtype: DTypeLike) -> str: ...
def check_variable_and_dtype(
    input: Tensor,
    input_name: str,
    expected_dtype: DTypeLike,
    op_name: str,
    extra_message: str = ...,
) -> None: ...
def check_type(
    input: Tensor,
    input_name: str,
    expected_type: type[Tensor],
    op_name: str,
    extra_message: str = ...,
) -> None: ...
def check_dtype(
    input_dtype: DTypeLike,
    input_name: str,
    expected_dtype: DTypeLike,
    op_name: str,
    extra_message: str = ...,
) -> None: ...
def check_shape(
    shape: ShapeLike,
    op_name: str,
    expected_shape_type: tuple[type[list] | type[tuple] | type[Variable], ...] = ...,
    expected_element_type: tuple[type[list] | type[tuple] | type[Variable], ...] = ...,
    expected_tensor_dtype: _DTypeString = ...,
) -> None: ...

class DataToLoDTensorConverter:
    place: Any = ...
    lod_level: Any = ...
    shape: Any = ...
    dtype: Any = ...
    def __init__(self, place: Any, lod_level: Any, shape: Any, dtype: Any) -> None: ...
    def feed(self, data: Any) -> None: ...
    def done(self): ...

class BatchedTensorProvider:
    place: Any = ...
    batch_size: Any = ...
    generator: Any = ...
    converters: Any = ...
    drop_last: Any = ...
    def __init__(self, feed_list: Any, place: Any, batch_size: Any, generator: Any, drop_last: Any) -> None: ...
    def __call__(self) -> None: ...

class DataFeeder:
    feed_dtypes: Any = ...
    feed_names: Any = ...
    feed_shapes: Any = ...
    feed_lod_level: Any = ...
    place: Any = ...
    def __init__(self, feed_list: Any, place: Any, program: Optional[Any] = ...) -> None: ...
    def feed(self, iterable: Any): ...
    def feed_parallel(self, iterable: Any, num_places: Optional[Any] = ...) -> None: ...
    def decorate_reader(
        self, reader: Any, multi_devices: Any, num_places: Optional[Any] = ..., drop_last: bool = ...
    ): ...



from __future__ import annotations

from paddle.fluid.framework import Parameter as Parameter

from ..base.private_helper_function import wait_server_ready as wait_server_ready
from .runtime_base import RuntimeBase as RuntimeBase

class ParameterServerRuntime(RuntimeBase):
    def __init__(self) -> None: ...
    def build_compiled_startegy(self): ...



from __future__ import annotations

from typing import Any

from .args_envs import env_args_mapping as env_args_mapping
from .args_envs import fetch_envs as fetch_envs
from .args_envs import parse_args as parse_args
from .node import Node as Node
from .status import Status as Status

class Context:
    envs: Any = ...
    node: Any = ...
    status: Any = ...
    logger: Any = ...
    events: Any = ...
    def __init__(self, enable_plugin: bool = ...) -> None: ...
    def is_legacy_mode(self): ...
    def get_envs(self): ...
    def get_logger(self, level: Any = ...): ...
    def continous_log(self) -> bool: ...
    def set_env_in_args(self) -> None: ...



from __future__ import annotations

from typing import Any, Optional

from ...device import get_cudnn_version as get_cudnn_version
from ...device import is_compiled_with_rocm as is_compiled_with_rocm
from ...fluid import dygraph_utils as dygraph_utils
from ...fluid.data_feeder import check_variable_and_dtype as check_variable_and_dtype
from ...fluid.layer_helper import LayerHelper as LayerHelper
from ...static import Variable as Variable

def affine_grid(theta: Any, out_shape: Any, align_corners: bool = ..., name: Optional[Any] = ...): ...
def grid_sample(
    x: Any, grid: Any, mode: str = ..., padding_mode: str = ..., align_corners: bool = ..., name: Optional[Any] = ...
): ...
def pixel_shuffle(x: Any, upscale_factor: Any, data_format: str = ..., name: Optional[Any] = ...): ...



from __future__ import annotations

from typing import Any, Optional

from paddle.fluid.framework import core as core

from ...fluid import dygraph_utils as dygraph_utils
from ...fluid.data_feeder import check_type as check_type
from ...fluid.data_feeder import check_variable_and_dtype as check_variable_and_dtype
from ...fluid.layer_helper import LayerHelper as LayerHelper
from ...framework import ParamAttr as ParamAttr
from ...framework import create_parameter as create_parameter
from ..initializer import Constant as Constant

def normalize(x: Any, p: int = ..., axis: int = ..., epsilon: float = ..., name: Optional[Any] = ...): ...
def batch_norm(
    x: Any,
    running_mean: Any,
    running_var: Any,
    weight: Any,
    bias: Any,
    training: bool = ...,
    momentum: float = ...,
    epsilon: float = ...,
    data_format: str = ...,
    use_global_stats: Optional[Any] = ...,
    name: Optional[Any] = ...,
): ...
def layer_norm(
    x: Any,
    normalized_shape: Any,
    weight: Optional[Any] = ...,
    bias: Optional[Any] = ...,
    epsilon: float = ...,
    name: Optional[Any] = ...,
): ...
def instance_norm(
    x: Any,
    running_mean: Optional[Any] = ...,
    running_var: Optional[Any] = ...,
    weight: Optional[Any] = ...,
    bias: Optional[Any] = ...,
    use_input_stats: bool = ...,
    momentum: float = ...,
    eps: float = ...,
    data_format: str = ...,
    name: Optional[Any] = ...,
): ...
def local_response_norm(
    x: Any,
    size: Any,
    alpha: float = ...,
    beta: float = ...,
    k: float = ...,
    data_format: str = ...,
    name: Optional[Any] = ...,
): ...
